"""配置导出/导入/备份/恢复 Mixin。

提供 ConfigManager 的配置数据导出、导入（合并/覆盖）、
文件备份及恢复能力。
"""

from __future__ import annotations

import json
import os
import tempfile
import time
from typing import TYPE_CHECKING, Any, cast

from enhanced_logging import EnhancedLogger

if TYPE_CHECKING:
    from pathlib import Path
    from threading import RLock

logger = EnhancedLogger(__name__)


class IOOperationsMixin:
    """配置导出/导入/备份/恢复方法集合。"""

    if TYPE_CHECKING:
        _lock: RLock
        _config: dict[str, Any]
        _pending_changes: dict[str, Any]
        config_file: Path
        _original_content: str | None

        def get_network_security_config(self) -> dict[str, Any]: ...
        def set_network_security_config(
            self,
            config: dict[str, Any],
            save: bool = True,
            trigger_callbacks: bool = True,
        ) -> None: ...
        @staticmethod
        def _exclude_network_security(config: dict[str, Any]) -> dict[str, Any]: ...
        def _save_config(self) -> None: ...
        def _trigger_config_change_callbacks(self) -> None: ...
        def _is_toml_file(self) -> bool: ...
        def _update_file_mtime(self) -> None: ...
        def _load_config(self) -> None: ...
        def invalidate_all_caches(self) -> None: ...

    def export_config(self, include_network_security: bool = False) -> dict[str, Any]:
        """导出当前配置（可选包含 network_security）"""
        with self._lock:
            export_data = {
                "exported_at": time.strftime("%Y-%m-%d %H:%M:%S"),
                "version": "1.0",
                "config": self._exclude_network_security(self._config.copy()),
            }

            if include_network_security:
                export_data["network_security"] = self.get_network_security_config()

            return export_data

    def import_config(
        self, config_data: dict[str, Any], merge: bool = True, save: bool = True
    ) -> bool:
        """导入配置（支持合并或覆盖模式）"""
        try:
            if not isinstance(config_data, dict):
                logger.error("导入失败：配置数据必须是字典格式")
                return False

            actual_config: Any
            network_security: dict[str, Any] | None = None

            if "config" in config_data:
                actual_config = config_data.get("config")
                network_security_raw = config_data.get("network_security")
                if isinstance(network_security_raw, dict):
                    network_security = cast(dict[str, Any], network_security_raw)
            else:
                actual_config = config_data
                network_security_raw = actual_config.get("network_security")
                if isinstance(network_security_raw, dict):
                    network_security = cast(dict[str, Any], network_security_raw)

            if not isinstance(actual_config, dict):
                logger.error("导入失败：配置数据必须是字典格式（config 字段）")
                return False

            if network_security is None:
                ns_in_config = actual_config.get("network_security")
                if isinstance(ns_in_config, dict):
                    network_security = cast(dict[str, Any], ns_in_config)

            with self._lock:
                if merge:
                    tmp = dict(actual_config)
                    tmp.pop("network_security", None)
                    self._deep_merge(self._config, tmp)
                    logger.info("配置已合并导入")
                else:
                    tmp = dict(actual_config)
                    tmp.pop("network_security", None)
                    self._config = tmp.copy()
                    logger.info("配置已覆盖导入")

                if save:
                    tmp = dict(actual_config)
                    tmp.pop("network_security", None)
                    self._pending_changes.update(tmp)
                    self._save_config()

            if network_security is not None:
                try:
                    self.set_network_security_config(
                        network_security, save=save, trigger_callbacks=False
                    )
                except Exception as e:
                    logger.error(f"导入 network_security 失败: {e}", exc_info=True)
                    return False

            self._trigger_config_change_callbacks()

            return True

        except Exception as e:
            logger.error(f"导入配置失败: {e}", exc_info=True)
            return False

    def _deep_merge(self, base: dict, update: dict):
        """递归合并 update 到 base"""
        for key, value in update.items():
            if key in base and isinstance(base[key], dict) and isinstance(value, dict):
                self._deep_merge(base[key], value)
            else:
                base[key] = value

    def backup_config(self, backup_path: str | None = None) -> str:
        """备份当前配置到文件"""
        if backup_path is None:
            backup_path = str(self.config_file) + ".backup"

        export_data = self.export_config(include_network_security=True)

        with open(backup_path, "w", encoding="utf-8") as f:
            json.dump(export_data, f, indent=2, ensure_ascii=False)

        logger.info(f"配置已备份到: {backup_path}")
        return backup_path

    def restore_config(self, backup_path: str) -> bool:
        """从备份文件恢复配置"""
        try:
            with open(backup_path, encoding="utf-8") as f:
                backup_data = json.load(f)

            if not isinstance(backup_data, dict):
                logger.error("恢复配置失败: 备份文件内容必须是字典")
                return False

            if "config" in backup_data:
                actual_config = backup_data.get("config", {})
                if not isinstance(actual_config, dict):
                    logger.error("恢复配置失败: 备份中的 config 必须是字典")
                    return False
                restored_config = dict(actual_config)
                network_security = backup_data.get("network_security")
                if isinstance(network_security, dict):
                    restored_config["network_security"] = network_security
            else:
                restored_config = dict(backup_data)

            self.config_file.parent.mkdir(parents=True, exist_ok=True)
            if self._is_toml_file():
                import tomlkit as _tk

                content = _tk.dumps(_tk.item(restored_config))
            else:
                content = json.dumps(restored_config, indent=2, ensure_ascii=False)

            # 原子写入：tempfile → fsync → os.replace
            fd, tmp_path = tempfile.mkstemp(
                suffix=".tmp", dir=str(self.config_file.parent)
            )
            try:
                with os.fdopen(fd, "w", encoding="utf-8") as f:
                    f.write(content)
                    f.flush()
                    os.fsync(f.fileno())
                os.replace(tmp_path, str(self.config_file))
            except BaseException:
                try:
                    os.unlink(tmp_path)
                except OSError:
                    pass
                raise

            with self._lock:
                self._original_content = content
            self._update_file_mtime()
            self._load_config()
            self.invalidate_all_caches()
            self._trigger_config_change_callbacks()
            logger.info(f"配置已从 {backup_path} 恢复")
            return True

        except FileNotFoundError:
            logger.error(f"备份文件不存在: {backup_path}")
            return False
        except json.JSONDecodeError as e:
            logger.error(f"备份文件格式错误: {e}", exc_info=True)
            return False
        except Exception as e:
            logger.error(f"恢复配置失败: {e}", exc_info=True)
            return False



"""config_modules — ConfigManager 路由 Mixin 模块。

将 ConfigManager 的方法按职责拆分为独立 Mixin，
降低单文件复杂度，同时保持外部 API 完全不变。
"""

from config_modules.file_watcher import FileWatcherMixin
from config_modules.io_operations import IOOperationsMixin
from config_modules.network_security import NetworkSecurityMixin
from config_modules.toml_engine import TomlEngineMixin

__all__ = [
    "FileWatcherMixin",
    "IOOperationsMixin",
    "NetworkSecurityMixin",
    "TomlEngineMixin",
]



"""反馈交互层 - interactive_feedback 工具实现、任务轮询与上下文管理。

该模块从 `server.py` 抽取反馈相关逻辑，避免在 MCP 入口文件里堆积业务代码。
注意：`interactive_feedback` 的 MCP 工具注册由 `server.py` 持有的 `mcp` 实例完成，
本模块内的 `interactive_feedback` 为“未装饰”的实现函数。
"""

from __future__ import annotations

import asyncio
import contextlib
import json
import time
from pathlib import Path
from typing import Any, cast

import httpx
from mcp.types import TextContent
from pydantic import Field

import server_config
import service_manager
from enhanced_logging import EnhancedLogger
from exceptions import (
    ServiceConnectionError,
    ServiceTimeoutError,
    ServiceUnavailableError,
    ValidationError,
)

logger = EnhancedLogger(__name__)


try:
    from notification_manager import NotificationTrigger, notification_manager
    from notification_models import NotificationType

    NOTIFICATION_AVAILABLE = True
    logger.info("通知系统已导入")
except ImportError as e:
    logger.warning(f"通知系统不可用: {e}", exc_info=True)
    NOTIFICATION_AVAILABLE = False


async def wait_for_task_completion(task_id: str, timeout: int = 260) -> dict[str, Any]:
    """SSE 事件驱动 + HTTP 轮询保底等待任务完成。

    双通道并行：SSE 提供 <50ms 实时检测，HTTP 轮询（每 2s）作为 SSE 断连的安全网。
    任一通道检测到完成即终止另一通道。
    """
    if timeout > 0:
        timeout = max(timeout, server_config.BACKEND_MIN)

    config, _ = service_manager.get_web_ui_config()
    target_host = server_config.get_target_host(config.host)
    api_url = f"http://{target_host}:{config.port}/api/tasks/{task_id}"
    sse_url = f"http://{target_host}:{config.port}/api/events"

    start_time_monotonic = time.monotonic()
    effective_timeout: float | None = float(timeout) if timeout > 0 else None

    logger.info(
        f"等待任务完成: {task_id}, "
        f"超时: {'无限等待' if timeout == 0 else f'{timeout}秒'}（SSE + 轮询）"
    )

    completion = asyncio.Event()
    result_box: list[Any] = [None]

    async def _fetch_result() -> dict[str, Any] | None:
        """获取已完成任务的结果，404 返回重调提示。"""
        try:
            cfg = service_manager.get_web_ui_config()[0]
            client = service_manager.get_async_client(cfg)
            resp = await client.get(api_url, timeout=2)
            if resp.status_code == 404:
                return server_config._make_resubmit_response(as_mcp=False)
            if resp.status_code == 200:
                data = resp.json()
                if data.get("success") and data.get("task"):
                    task = data["task"]
                    if (
                        isinstance(task, dict)
                        and task.get("status") == "completed"
                        and task.get("result")
                    ):
                        return task["result"]
        except Exception as e:
            logger.debug(f"获取任务结果失败: {e}")
        return None

    async def _sse_listener() -> None:
        """SSE 实时通道：收到 task_changed(completed) 即通知完成。"""
        try:
            async with (
                httpx.AsyncClient() as sc,
                sc.stream(
                    "GET", sse_url, timeout=httpx.Timeout(None, connect=5.0)
                ) as resp,
            ):
                logger.debug(f"SSE 连接已建立: {task_id}")
                async for line in resp.aiter_lines():
                    if completion.is_set():
                        return
                    stripped = line.strip()
                    if not stripped.startswith("data: "):
                        continue
                    try:
                        ev = json.loads(stripped[6:])
                    except (json.JSONDecodeError, ValueError):
                        continue
                    if (
                        ev.get("task_id") == task_id
                        and ev.get("new_status") == "completed"
                    ):
                        logger.info(f"SSE 检测到任务完成: {task_id}")
                        r = await _fetch_result()
                        if r is not None:
                            result_box[0] = r
                        completion.set()
                        return
        except asyncio.CancelledError:
            raise
        except Exception as e:
            logger.debug(f"SSE 监听失败（依赖轮询保底）: {e}")

    async def _poll_fallback() -> None:
        """HTTP 轮询保底：每 2s 检查一次，SSE 断开时仍能检测完成。"""
        _INTERVAL = 2.0
        while not completion.is_set():
            r = await _fetch_result()
            if r is not None:
                result_box[0] = r
                completion.set()
                return
            try:
                await asyncio.wait_for(completion.wait(), timeout=_INTERVAL)
                return
            except TimeoutError:
                pass

    sse_task = asyncio.create_task(_sse_listener())
    poll_task = asyncio.create_task(_poll_fallback())

    try:
        await asyncio.wait_for(completion.wait(), timeout=effective_timeout)
    except TimeoutError:
        elapsed = time.monotonic() - start_time_monotonic
        logger.error(f"任务超时: {task_id}, 等待 {elapsed:.1f}s")
        return cast(dict[str, Any], server_config._make_resubmit_response(as_mcp=False))
    finally:
        sse_task.cancel()
        poll_task.cancel()
        with contextlib.suppress(asyncio.CancelledError):
            await sse_task
        with contextlib.suppress(asyncio.CancelledError):
            await poll_task

    if result_box[0] is not None:
        logger.info(f"任务完成: {task_id}")
        return cast(dict[str, Any], result_box[0])

    r = await _fetch_result()
    if r is not None:
        return r

    return cast(dict[str, Any], server_config._make_resubmit_response(as_mcp=False))


def launch_feedback_ui(
    summary: str,
    predefined_options: list[str] | None = None,
    task_id: str | None = None,
    timeout: int = 300,
) -> dict[str, Any]:
    """废弃：旧版 Python API，推荐使用 interactive_feedback() MCP 工具"""
    # 确保超时时间不小于300秒（0表示无限等待，保持不变）
    if timeout > 0:
        timeout = max(timeout, 300)
    try:
        # 自动生成唯一 task_id（task_id 参数将被忽略，始终使用自动生成）
        task_id = server_config._generate_task_id()

        # 验证输入参数
        cleaned_summary, cleaned_options = server_config.validate_input(
            summary, predefined_options
        )

        # 获取配置
        config, auto_resubmit_timeout = service_manager.get_web_ui_config()

        logger.info(
            f"启动反馈界面: {cleaned_summary[:100]}... (自动生成task_id: {task_id})"
        )

        # 确保 Web UI 正在运行（在同步函数中运行异步函数）
        asyncio.run(service_manager.ensure_web_ui_running(config))

        # 通过 HTTP API 向 web_ui 添加任务
        target_host = server_config.get_target_host(config.host)
        api_url = f"http://{target_host}:{config.port}/api/tasks"

        try:
            client = service_manager.get_sync_client(config)
            response = client.post(
                api_url,
                json={
                    "task_id": task_id,
                    "prompt": cleaned_summary,
                    "predefined_options": cleaned_options,
                    "auto_resubmit_timeout": auto_resubmit_timeout,
                },
                timeout=5,
            )

            if response.status_code != 200:
                error_detail = "未知错误"
                try:
                    payload = response.json()
                    if isinstance(payload, dict):
                        error_detail = str(payload.get("error", error_detail))
                    else:
                        error_detail = str(payload)
                except ValueError:
                    try:
                        if response.text:
                            error_detail = response.text[:200]
                    except Exception:
                        pass
                logger.error(
                    f"添加任务失败: HTTP {response.status_code}, 详情: {error_detail}"
                )
                return {
                    "error": f"添加任务失败: {error_detail}",
                }

            logger.info(f"任务已通过API添加到队列: {task_id}")

            # 【新增】发送通知（立即触发，不阻塞主流程）
            if NOTIFICATION_AVAILABLE:
                try:
                    # 【关键修复】从配置文件刷新配置，解决跨进程配置不同步问题
                    # Web UI 以子进程方式运行，配置更新只发生在 Web UI 进程中
                    # MCP 服务器进程需要在发送通知前同步最新配置
                    notification_manager.refresh_config_from_file()

                    # 截断消息，避免过长（Bark 有长度限制）
                    notification_message = cleaned_summary[:100]
                    if len(cleaned_summary) > 100:
                        notification_message += "..."

                    # MCP 侧仅发送系统级通知，Bark 由前端统一处理（避免跨进程双重推送）
                    mcp_types = [
                        t
                        for t in [NotificationType.SYSTEM, NotificationType.SOUND]
                        if t != NotificationType.BARK
                    ]
                    event_id = notification_manager.send_notification(
                        title="新的交互反馈请求",
                        message=notification_message,
                        trigger=NotificationTrigger.IMMEDIATE,
                        types=mcp_types,
                        metadata={"task_id": task_id, "source": "launch_feedback_ui"},
                    )

                    if event_id:
                        logger.debug(
                            f"已为任务 {task_id} 发送通知，事件 ID: {event_id}"
                        )
                    else:
                        logger.debug(f"任务 {task_id} 通知已跳过（通知系统已禁用）")

                except Exception as e:
                    # 通知失败不影响任务创建，仅记录警告
                    logger.warning(
                        f"发送任务通知失败: {e}，任务 {task_id} 已正常创建",
                        exc_info=True,
                    )
            else:
                logger.debug("通知系统不可用，跳过通知发送")

        except httpx.HTTPError as e:
            logger.error(f"添加任务请求失败: {e}", exc_info=True)
            return {
                "error": f"无法连接到 Web UI：{e}。请确认 Web UI 服务已启动，并检查地址/端口配置（如 web_ui.host/web_ui.port 或 VS Code 的 serverUrl）。"
            }

        # 【优化】使用统一的超时计算函数
        # timeout=0 表示无限等待模式
        backend_timeout = server_config.calculate_backend_timeout(
            auto_resubmit_timeout,
            max_timeout=max(timeout, 0),  # 传入的 timeout 参数作为参考
            infinite_wait=(timeout == 0),
        )
        logger.info(
            f"后端等待时间: {backend_timeout}秒 (前端倒计时: {auto_resubmit_timeout}秒, 传入timeout: {timeout}秒)"
        )
        # 在同步函数中运行异步函数（废弃的 API，保持向后兼容）
        result = asyncio.run(wait_for_task_completion(task_id, timeout=backend_timeout))

        if "error" in result:
            logger.error(f"任务执行失败: {result['error']}")
            return {"error": result["error"]}

        logger.info("用户反馈收集完成")
        return result

    except ValueError as e:
        logger.error(f"输入参数错误: {e}", exc_info=True)
        raise ValidationError(f"参数验证失败: {e}", code="invalid_params") from e
    except FileNotFoundError as e:
        logger.error(f"文件未找到: {e}", exc_info=True)
        raise ServiceUnavailableError(
            f"必要文件缺失: {e}", code="file_not_found"
        ) from e
    except (
        ServiceConnectionError,
        ServiceTimeoutError,
        ServiceUnavailableError,
        ValidationError,
    ):
        raise
    except Exception as e:
        logger.error(f"启动反馈界面失败: {e}", exc_info=True)
        raise ServiceUnavailableError(
            f"反馈界面启动失败: {e}", code="start_failed"
        ) from e


async def interactive_feedback(
    message: str = Field(description="向用户展示的具体问题/提示（支持 Markdown）"),
    predefined_options: list | None = Field(
        default=None,
        description="可选的预定义选项列表，供用户单选/多选",
    ),
) -> list:
    """
    MCP 工具实现：请求用户通过 Web UI 提供交互反馈

    注意：该函数本身不负责 MCP 工具注册；注册由 `server.py` 中的 `mcp` 完成。
    """
    try:
        # 输入清理：截断过长内容，过滤非法选项（对齐工具契约/避免后端 400）
        cleaned_message, cleaned_options = server_config.validate_input(
            message, predefined_options
        )
        predefined_options_list = cleaned_options

        # 自动生成唯一 task_id（避免极端并发下碰撞）
        task_id = server_config._generate_task_id()

        logger.info(
            f"收到反馈请求: {cleaned_message[:50]}... (自动生成task_id: {task_id})"
        )

        # 获取配置
        config, auto_resubmit_timeout = service_manager.get_web_ui_config()

        # 确保 Web UI 正在运行
        await service_manager.ensure_web_ui_running(config)

        # 通过 HTTP API 添加任务
        target_host = server_config.get_target_host(config.host)
        api_url = f"http://{target_host}:{config.port}/api/tasks"

        try:
            client = service_manager.get_async_client(config)
            response = await client.post(
                api_url,
                json={
                    "task_id": task_id,
                    "prompt": cleaned_message,
                    "predefined_options": predefined_options_list,
                    "auto_resubmit_timeout": auto_resubmit_timeout,
                },
                timeout=5,
            )

            if response.status_code != 200:
                # 记录详细错误信息到日志
                error_detail = "未知错误"
                try:
                    payload = response.json()
                    if isinstance(payload, dict):
                        error_detail = str(payload.get("error", error_detail))
                    else:
                        error_detail = str(payload)
                except ValueError as e:
                    logger.warning(
                        f"添加任务失败响应不是有效 JSON: {e}",
                        exc_info=True,
                    )
                    try:
                        if response.text:
                            error_detail = response.text[:200]
                    except Exception:
                        # response.text 读取失败不应影响主流程
                        pass
                logger.error(
                    f"添加任务失败: HTTP {response.status_code}, 详情: {error_detail}"
                )
                # 返回配置的提示语，引导 AI 重新调用工具
                return server_config._make_resubmit_response()

            logger.info(f"任务已通过API添加到队列: {task_id}")

            # 【新增】发送通知（立即触发，不阻塞主流程）
            if NOTIFICATION_AVAILABLE:
                try:
                    # 【关键修复】从配置文件刷新配置，解决跨进程配置不同步问题
                    # Web UI 以子进程方式运行，配置更新只发生在 Web UI 进程中
                    # MCP 服务器进程需要在发送通知前同步最新配置
                    notification_manager.refresh_config_from_file()

                    # 截断消息，避免过长（Bark 有长度限制）
                    notification_message = cleaned_message[:100]
                    if len(cleaned_message) > 100:
                        notification_message += "..."

                    # MCP 侧仅发送系统级通知，Bark 由前端统一处理（避免跨进程双重推送）
                    mcp_types = [
                        t
                        for t in [NotificationType.SYSTEM, NotificationType.SOUND]
                        if t != NotificationType.BARK
                    ]
                    event_id = notification_manager.send_notification(
                        title="新的反馈请求",
                        message=notification_message,
                        trigger=NotificationTrigger.IMMEDIATE,
                        types=mcp_types,
                        metadata={"task_id": task_id, "source": "interactive_feedback"},
                    )

                    if event_id:
                        logger.debug(
                            f"已为任务 {task_id} 发送通知，事件 ID: {event_id}"
                        )
                    else:
                        logger.debug(f"任务 {task_id} 通知已跳过（通知系统已禁用）")

                except Exception as e:
                    # 通知失败不影响任务创建，仅记录警告
                    logger.warning(
                        f"发送任务通知失败: {e}，任务 {task_id} 已正常创建",
                        exc_info=True,
                    )
            else:
                logger.debug("通知系统不可用，跳过通知发送")

        except httpx.HTTPError as e:
            logger.error(f"添加任务请求失败，无法连接到 Web UI: {e}", exc_info=True)
            # 返回配置的提示语，引导 AI 重新调用工具
            return server_config._make_resubmit_response()

        # 【优化】使用统一的超时计算函数，利用 feedback.timeout 作为上限
        backend_timeout = server_config.calculate_backend_timeout(auto_resubmit_timeout)
        logger.info(
            f"后端等待时间: {backend_timeout}秒 (前端倒计时: {auto_resubmit_timeout}秒)"
        )
        result = await wait_for_task_completion(task_id, timeout=backend_timeout)

        if "error" in result:
            # 记录任务执行失败的详细错误
            logger.error(f"任务执行失败: {result['error']}, 任务 ID: {task_id}")
            # 返回配置的提示语，引导 AI 重新调用工具
            return server_config._make_resubmit_response()

        logger.info("反馈请求处理完成")

        # 解析返回：兼容新旧格式 + 兜底处理 {"text": "..."} 降级返回
        if isinstance(result, dict):
            # wait_for_task_completion 的降级返回（超时/404）：{"text": "..."}
            if set(result.keys()) == {"text"} and isinstance(result.get("text"), str):
                return [TextContent(type="text", text=str(result["text"]))]

            # 新格式（结构化 JSON，可能含 images）
            if (
                "images" in result
                or "user_input" in result
                or "selected_options" in result
            ):
                return server_config.parse_structured_response(result)

            # 旧格式：只有文本反馈
            legacy = result.get("interactive_feedback")
            if isinstance(legacy, str) and legacy.strip():
                return [
                    TextContent(
                        type="text",
                        text=server_config._append_prompt_suffix(legacy),
                    )
                ]

            # 最后兜底：尽量取 text 字段，否则转字符串
            fallback = (
                result.get("text")
                if isinstance(result.get("text"), str)
                else str(result)
            )
            return [
                TextContent(
                    type="text",
                    text=server_config._append_prompt_suffix(str(fallback)),
                )
            ]

        # 简单字符串结果
        return [
            TextContent(
                type="text",
                text=server_config._append_prompt_suffix(str(result)),
            )
        ]

    except Exception as e:
        logger.error(f"interactive_feedback 工具执行失败: {e}", exc_info=True)
        # 返回配置的提示语，引导 AI 重新调用工具
        return server_config._make_resubmit_response()


class FeedbackServiceContext:
    """反馈服务上下文管理器 - 自动管理服务启动和清理"""

    def __init__(self):
        """初始化，延迟加载配置"""
        self.service_manager = service_manager.ServiceManager()
        self.config = None
        self.script_dir = None

    def __enter__(self):
        """加载配置并返回 self"""
        try:
            self.config, self.auto_resubmit_timeout = (
                service_manager.get_web_ui_config()
            )
            self.script_dir = Path(__file__).resolve().parent
            logger.info(
                f"反馈服务上下文已初始化，自动重调超时: {self.auto_resubmit_timeout}秒"
            )
            return self
        except Exception as e:
            logger.error(f"初始化反馈服务上下文失败: {e}", exc_info=True)
            raise

    def __exit__(self, exc_type, exc_val, exc_tb):
        """清理所有服务进程（退出上下文时）"""
        del exc_tb
        try:
            # 上下文退出不等同于进程退出：仅清理子进程/端口等资源，保留通知线程池可用性
            self.service_manager.cleanup_all(shutdown_notification_manager=False)
            if exc_type is KeyboardInterrupt:
                logger.info("收到中断信号，服务已清理")
            elif exc_type is not None:
                logger.error(f"异常退出，服务已清理: {exc_type.__name__}: {exc_val}")
            else:
                logger.info("正常退出，服务已清理")
        except Exception as e:
            logger.error(f"清理服务时出错: {e}", exc_info=True)

    def launch_feedback_ui(
        self,
        summary: str,
        predefined_options: list[str] | None = None,
        task_id: str | None = None,
        timeout: int = 300,
    ) -> dict[str, Any]:
        """在上下文中启动反馈界面（委托给全局 launch_feedback_ui）"""
        return launch_feedback_ui(summary, predefined_options, task_id, timeout)



"""
共享类型定义（Pydantic 配置段模型 + TypedDict 反馈结构）

目的：
- 配置段模型：提供 TOML 配置段的运行时校验与类型安全
- TypedDict：让 `ty` 在跨模块分析时拥有一致的结构化类型

命名规则：
- 配置段模型以 `SectionConfig` 后缀命名，与 notification_manager.NotificationConfig 等运行时模型区分
"""

from typing import Annotated, Any, TypedDict

from pydantic import BaseModel, BeforeValidator, ConfigDict

# ---------------------------------------------------------------------------
# 可复用的 Pydantic 前置校验器（替代 safe_bool / safe_int / safe_str）
# ---------------------------------------------------------------------------


def _coerce_bool(v: Any) -> Any:
    """TOML/JSON 安全布尔值转换（兼容字符串 "true"/"false"/数字 0/1）"""
    if isinstance(v, bool):
        return v
    if isinstance(v, (int, float)):
        return bool(v)
    if isinstance(v, str):
        s = v.strip().lower()
        if s in ("true", "1", "yes", "y", "on"):
            return True
        if s in ("false", "0", "no", "n", "off"):
            return False
    return v


def _coerce_int(v: Any) -> Any:
    """TOML/JSON 安全整数转换（兼容浮点数和数字字符串）"""
    if isinstance(v, bool):
        return int(v)
    if isinstance(v, int):
        return v
    try:
        return int(float(v))
    except (TypeError, ValueError):
        return v


def _clamp_int(min_val: int, max_val: int, default: int):
    """生成一个 BeforeValidator：将整数值钳位到 [min_val, max_val]，失败返回 default"""

    def _validator(v: Any) -> int:
        try:
            iv = int(float(v)) if not isinstance(v, bool) else int(v)
        except (TypeError, ValueError):
            return default
        return max(min_val, min(max_val, iv))

    return _validator


def _clamp_int_allow_zero(min_val: int, max_val: int, default: int):
    """生成一个 BeforeValidator：0 或负值 → 0（禁用），其余钳位到 [min_val, max_val]"""

    def _validator(v: Any) -> int:
        try:
            iv = int(float(v)) if not isinstance(v, bool) else int(v)
        except (TypeError, ValueError):
            return default
        if iv <= 0:
            return 0
        return max(min_val, min(max_val, iv))

    return _validator


def _clamp_float(min_val: float, max_val: float, default: float):
    """生成一个 BeforeValidator：将浮点值钳位到 [min_val, max_val]，失败返回 default"""

    def _validator(v: Any) -> float:
        try:
            fv = float(v)
        except (TypeError, ValueError):
            return default
        return max(min_val, min(max_val, fv))

    return _validator


def _coerce_float(v: Any) -> Any:
    """TOML/JSON 安全浮点转换"""
    if isinstance(v, (int, float)) and not isinstance(v, bool):
        return float(v)
    try:
        return float(v)
    except (TypeError, ValueError):
        return v


def _coerce_str(v: Any) -> Any:
    """TOML/JSON 安全字符串转换（None → 默认由 Pydantic 处理）"""
    if v is None:
        return v
    return str(v)


SafeBool = Annotated[bool, BeforeValidator(_coerce_bool)]
SafeInt = Annotated[int, BeforeValidator(_coerce_int)]
SafeFloat = Annotated[float, BeforeValidator(_coerce_float)]
SafeStr = Annotated[str, BeforeValidator(_coerce_str)]


# ---------------------------------------------------------------------------
# 反馈数据结构（TypedDict，用于运行时字典构造）
# ---------------------------------------------------------------------------


class FeedbackImage(TypedDict, total=False):
    """单张图片的结构（Web UI / MCP 交互中使用）"""

    data: str
    filename: str
    size: int
    content_type: str
    mimeType: str
    mime_type: str


class FeedbackResult(TypedDict):
    """Web UI 反馈结果结构（与 /api/feedback 返回一致）"""

    user_input: str
    selected_options: list[str]
    images: list[FeedbackImage]


# ---------------------------------------------------------------------------
# Pydantic 配置段模型（与 config_manager._get_default_config 对齐）
#
# 每个模型的默认值即为 TOML 配置的默认值（单一真相源）。
# extra="allow" 确保未知键不会被丢弃（用户可自定义扩展键）。
# ---------------------------------------------------------------------------


class NotificationSectionConfig(BaseModel):
    """notification TOML 配置段（与 notification_manager.NotificationConfig 区分）"""

    model_config = ConfigDict(extra="allow")

    enabled: SafeBool = True
    debug: SafeBool = False
    web_enabled: SafeBool = True
    auto_request_permission: SafeBool = True
    web_icon: SafeStr = "default"
    web_timeout: Annotated[int, BeforeValidator(_clamp_int(1, 600000, 5000))] = 5000
    system_enabled: SafeBool = False
    macos_native_enabled: SafeBool = True
    sound_enabled: SafeBool = True
    sound_mute: SafeBool = False
    sound_file: SafeStr = "default"
    sound_volume: Annotated[int, BeforeValidator(_clamp_int(0, 100, 80))] = 80
    mobile_optimized: SafeBool = True
    mobile_vibrate: SafeBool = True
    retry_count: Annotated[int, BeforeValidator(_clamp_int(0, 10, 3))] = 3
    retry_delay: Annotated[int, BeforeValidator(_clamp_int(0, 60, 2))] = 2
    bark_enabled: SafeBool = False
    bark_url: SafeStr = ""
    bark_device_key: SafeStr = ""
    bark_icon: SafeStr = ""
    bark_action: SafeStr = "none"
    bark_timeout: Annotated[int, BeforeValidator(_clamp_int(1, 300, 10))] = 10


class WebUISectionConfig(BaseModel):
    """web_ui TOML 配置段"""

    model_config = ConfigDict(extra="allow")

    host: SafeStr = "127.0.0.1"
    port: Annotated[int, BeforeValidator(_clamp_int(1, 65535, 8080))] = 8080
    language: SafeStr = "auto"
    debug: SafeBool = False
    http_request_timeout: Annotated[int, BeforeValidator(_clamp_int(1, 600, 30))] = 30
    http_max_retries: Annotated[int, BeforeValidator(_clamp_int(0, 20, 3))] = 3
    http_retry_delay: Annotated[float, BeforeValidator(_clamp_float(0, 60, 1.0))] = 1.0


class MdnsSectionConfig(BaseModel):
    """mdns TOML 配置段"""

    model_config = ConfigDict(extra="allow")

    enabled: SafeBool | str = "auto"
    hostname: SafeStr = "ai.local"
    service_name: SafeStr = "AI Intervention Agent"


class NetworkSecuritySectionConfig(BaseModel):
    """network_security TOML 配置段"""

    model_config = ConfigDict(extra="allow")

    bind_interface: SafeStr = "0.0.0.0"
    allowed_networks: list[str] = [
        "127.0.0.0/8",
        "::1/128",
        "192.168.0.0/16",
        "10.0.0.0/8",
        "172.16.0.0/12",
    ]
    blocked_ips: list[str] = []
    access_control_enabled: SafeBool = True


class FeedbackSectionConfig(BaseModel):
    """feedback TOML 配置段"""

    model_config = ConfigDict(extra="allow")

    backend_max_wait: Annotated[int, BeforeValidator(_clamp_int(10, 7200, 600))] = 600
    frontend_countdown: Annotated[
        int, BeforeValidator(_clamp_int_allow_zero(10, 3600, 240))
    ] = 240
    resubmit_prompt: SafeStr = "请立即调用 interactive_feedback 工具"
    prompt_suffix: SafeStr = "\n请积极调用 interactive_feedback 工具"


# ---------------------------------------------------------------------------
# 段名 → 模型类 注册表（供 config_manager 使用）
# ---------------------------------------------------------------------------

SECTION_MODELS: dict[str, type[BaseModel]] = {
    "notification": NotificationSectionConfig,
    "web_ui": WebUISectionConfig,
    "mdns": MdnsSectionConfig,
    "network_security": NetworkSecuritySectionConfig,
    "feedback": FeedbackSectionConfig,
}

# ---------------------------------------------------------------------------
# 向后兼容别名（旧名 → 新名，逐步淘汰）
# ---------------------------------------------------------------------------
NotificationConfig = NotificationSectionConfig
MdnsConfig = MdnsSectionConfig
NetworkSecurityConfig = NetworkSecuritySectionConfig
FeedbackConfig = FeedbackSectionConfig



"""Network Security 配置管理 Mixin。

提供 ConfigManager 中与 network_security 段相关的
校验、读取、写入、增量更新能力。
"""

from __future__ import annotations

import json
import os
import tempfile
import time
from ipaddress import AddressValueError, ip_address, ip_network
from typing import TYPE_CHECKING, Any, cast

from enhanced_logging import EnhancedLogger

if TYPE_CHECKING:
    from pathlib import Path
    from threading import RLock

logger = EnhancedLogger(__name__)


class NetworkSecurityMixin:
    """network_security 配置段的校验/读写/缓存管理。"""

    if TYPE_CHECKING:
        _lock: RLock
        config_file: Path
        _original_content: str | None
        _network_security_cache: dict[str, Any] | None
        _network_security_cache_time: float
        _network_security_cache_ttl: float

        def _get_default_config(self) -> dict[str, Any]: ...
        @staticmethod
        def _coerce_bool(value: Any, default: bool = True) -> bool: ...
        def _create_default_config_file(self) -> None: ...
        def _is_toml_file(self) -> bool: ...
        def _save_network_security_toml(self, ns_config: dict[str, Any]) -> str: ...
        def _update_file_mtime(self) -> None: ...
        def invalidate_all_caches(self) -> None: ...
        def _trigger_config_change_callbacks(self) -> None: ...
        def _parse_config_content(self, content: str) -> dict[str, Any]: ...
        def _validate_config_structure(
            self, parsed_config: dict[str, Any], content: str
        ) -> None: ...

    def _validate_network_security_config(self, raw: Any) -> dict[str, Any]:
        """强校验并归一化 network_security（与文档/模板对齐，兼容旧字段）"""
        default_ns = cast(
            dict[str, Any],
            self._get_default_config().get("network_security", {}),
        )

        if not isinstance(raw, dict):
            raw = {}

        bind_raw = raw.get(
            "bind_interface", default_ns.get("bind_interface", "0.0.0.0")
        )
        bind = "127.0.0.1"
        try:
            if isinstance(bind_raw, str):
                s = bind_raw.strip()
                if s in ("0.0.0.0", "127.0.0.1", "localhost", "::1", "::"):
                    bind = s
                else:
                    bind = str(ip_address(s))
            else:
                bind = str(ip_address(str(bind_raw).strip()))
        except Exception:
            logger.warning(
                f"network_security.bind_interface 无效，回退为 127.0.0.1: {bind_raw}"
            )
            bind = "127.0.0.1"

        def _dedupe_keep_order(items: list[str]) -> list[str]:
            seen: set[str] = set()
            out: list[str] = []
            for x in items:
                if x in seen:
                    continue
                seen.add(x)
                out.append(x)
            return out

        allowed_raw = raw.get(
            "allowed_networks", default_ns.get("allowed_networks", [])
        )
        allowed_list: list[str] = []
        if isinstance(allowed_raw, list):
            for item in allowed_raw:
                if not isinstance(item, str):
                    continue
                t = item.strip()
                if not t:
                    continue
                try:
                    if "/" in t:
                        allowed_list.append(str(ip_network(t, strict=False)))
                    else:
                        allowed_list.append(str(ip_address(t)))
                except Exception:
                    logger.warning(f"allowed_networks 无效条目已忽略: {t}")
        else:
            logger.warning("allowed_networks 不是列表，使用默认值")
            allowed_list = []

        allowed_list = _dedupe_keep_order(allowed_list)
        if not allowed_list:
            allowed_list = ["127.0.0.0/8", "::1/128"]

        blocked_raw = raw.get("blocked_ips", default_ns.get("blocked_ips", []))
        blocked_list: list[str] = []
        if isinstance(blocked_raw, list):
            for item in blocked_raw:
                if not isinstance(item, str):
                    continue
                t = item.strip()
                if not t:
                    continue
                try:
                    if "/" in t:
                        blocked_list.append(str(ip_network(t, strict=False)))
                    else:
                        blocked_list.append(str(ip_address(t)))
                except (AddressValueError, Exception):
                    logger.warning(f"blocked_ips 无效条目已忽略: {t}")
        else:
            logger.warning("blocked_ips 不是列表，使用默认值")
            blocked_list = []

        blocked_list = _dedupe_keep_order(blocked_list)

        access_enabled = self._coerce_bool(
            raw.get(
                "access_control_enabled",
                raw.get(
                    "enable_access_control",
                    default_ns.get("access_control_enabled", True),
                ),
            ),
            default=True,
        )

        return {
            "bind_interface": bind,
            "allowed_networks": allowed_list,
            "blocked_ips": blocked_list,
            "access_control_enabled": access_enabled,
        }

    def _atomic_write_config(self, new_content: str) -> None:
        """原子写入配置文件（tempfile + os.replace），与 _save_config_immediate 保持一致"""
        self.config_file.parent.mkdir(parents=True, exist_ok=True)

        orig_mode = None
        if hasattr(os, "fchmod"):
            try:
                orig_mode = os.stat(str(self.config_file)).st_mode
            except OSError:
                pass

        fd, tmp_path = tempfile.mkstemp(
            suffix=".tmp",
            dir=str(self.config_file.parent),
        )
        try:
            if orig_mode is not None:
                os.fchmod(fd, orig_mode)
            with os.fdopen(fd, "w", encoding="utf-8") as f:
                f.write(new_content)
                f.flush()
                os.fsync(f.fileno())
            os.replace(tmp_path, str(self.config_file))
        except BaseException:
            try:
                os.unlink(tmp_path)
            except OSError:
                pass
            raise

    def _save_network_security_config_immediate(self, validated_ns: dict[str, Any]):
        """将 network_security 原子写回配置文件（不走通用保存逻辑，避免被排除）"""
        try:
            if not self.config_file.exists():
                self._create_default_config_file()
        except Exception:
            pass

        content = ""
        try:
            if self.config_file.exists():
                content = self.config_file.read_text(encoding="utf-8")
        except Exception as e:
            raise RuntimeError(f"读取配置文件失败: {e}") from e

        # TOML 格式
        if self._is_toml_file():
            base = content or (self._original_content or "")
            if base:
                new_content = self._save_network_security_toml(validated_ns)
            else:
                import tomlkit as _tk

                doc = _tk.document()
                doc["network_security"] = validated_ns
                new_content = _tk.dumps(doc)
            try:
                self._atomic_write_config(new_content)
            except Exception as e:
                raise RuntimeError(f"写入配置文件失败: {e}") from e
            with self._lock:
                self._original_content = new_content
            self._update_file_mtime()
            return

        # JSON 格式（降级兼容）
        try:
            full = json.loads(content) if content.strip() else {}
            if not isinstance(full, dict):
                full = {}
        except Exception:
            full = {}
        full["network_security"] = validated_ns
        new_content = json.dumps(full, indent=2, ensure_ascii=False)
        try:
            self._atomic_write_config(new_content)
        except Exception as e:
            raise RuntimeError(f"写入配置文件失败: {e}") from e
        with self._lock:
            self._original_content = new_content
        self._update_file_mtime()

    def set_network_security_config(
        self, config: dict[str, Any], save: bool = True, trigger_callbacks: bool = True
    ) -> None:
        """设置并持久化 network_security（强校验 + 单一路径写回）"""
        validated = self._validate_network_security_config(config)
        if save:
            self._save_network_security_config_immediate(validated)
        with self._lock:
            self._network_security_cache = validated
            self._network_security_cache_time = time.monotonic()
        self.invalidate_all_caches()
        if trigger_callbacks:
            try:
                self._trigger_config_change_callbacks()
            except Exception as e:
                logger.debug(f"触发配置变更回调失败（忽略）: {e}")

    def update_network_security_config(
        self, updates: dict[str, Any], save: bool = True, trigger_callbacks: bool = True
    ) -> None:
        """增量更新并持久化 network_security（只允许白名单字段）"""
        if not isinstance(updates, dict):
            raise ValueError("network_security 更新必须是 dict")

        current = self.get_network_security_config()
        merged = dict(current)

        for k, v in updates.items():
            if k in ("bind_interface", "allowed_networks", "blocked_ips"):
                merged[k] = v
            elif k in ("access_control_enabled", "enable_access_control"):
                merged["access_control_enabled"] = v
            else:
                logger.warning(f"忽略未知的 network_security 字段: {k}")

        validated = self._validate_network_security_config(merged)
        if save:
            self._save_network_security_config_immediate(validated)

        with self._lock:
            self._network_security_cache = validated
            self._network_security_cache_time = time.monotonic()

        self.invalidate_all_caches()
        if trigger_callbacks:
            try:
                self._trigger_config_change_callbacks()
            except Exception as e:
                logger.debug(f"触发配置变更回调失败（忽略）: {e}")

    def get_network_security_config(self) -> dict[str, Any]:
        """从文件读取 network_security 配置（带 30 秒缓存，失败返回默认配置）"""
        current_time = time.monotonic()
        with self._lock:
            if (
                self._network_security_cache is not None
                and current_time - self._network_security_cache_time
                < self._network_security_cache_ttl
            ):
                logger.debug("使用缓存的 network_security 配置")
                return self._network_security_cache

        try:
            if not self.config_file.exists():
                default_config = self._get_default_config()
                raw_result = cast(
                    dict[str, Any], default_config.get("network_security", {})
                )
                result = self._validate_network_security_config(raw_result)
                with self._lock:
                    self._network_security_cache = result
                    self._network_security_cache_time = current_time
                return result

            with open(self.config_file, encoding="utf-8") as f:
                content = f.read()

            full_config = self._parse_config_content(content)

            self._validate_config_structure(full_config, content)

            network_security_config = cast(
                dict[str, Any], full_config.get("network_security", {})
            )

            if not network_security_config:
                default_config = self._get_default_config()
                network_security_config = cast(
                    dict[str, Any], default_config.get("network_security", {})
                )
                logger.debug("配置文件中未找到network_security，使用默认配置")

            validated = self._validate_network_security_config(network_security_config)

            with self._lock:
                self._network_security_cache = validated
                self._network_security_cache_time = current_time
                logger.debug("已更新 network_security 配置缓存")

            return validated

        except Exception as e:
            logger.error(f"读取 network_security 配置失败: {e}", exc_info=True)
            with self._lock:
                if self._network_security_cache is not None:
                    logger.warning(
                        "读取 network_security 配置失败，返回缓存的上一次成功配置",
                        exc_info=True,
                    )
                    return self._network_security_cache

            default_config = self._get_default_config()
            raw_default = cast(
                dict[str, Any], default_config.get("network_security", {})
            )
            return self._validate_network_security_config(raw_default)



"""配置文件监听 Mixin。

提供 ConfigManager 的后台文件监听、配置变更回调注册/触发、
以及 shutdown 生命周期管理能力。
"""

from __future__ import annotations

import threading
from collections.abc import Callable
from typing import TYPE_CHECKING

from enhanced_logging import EnhancedLogger

if TYPE_CHECKING:
    from pathlib import Path

logger = EnhancedLogger(__name__)


class FileWatcherMixin:
    """文件监听、回调管理、shutdown 方法集合。"""

    if TYPE_CHECKING:
        config_file: Path
        _lock: threading.RLock
        _last_file_mtime: float
        _file_watcher_running: bool
        _file_watcher_interval: float
        _file_watcher_stop_event: threading.Event
        _file_watcher_thread: threading.Thread | None
        _save_timer: threading.Timer | None
        _config_change_callbacks: list[Callable[[], None]]

        def reload(self) -> None: ...
        def force_save(self) -> None: ...

    def _update_file_mtime(self):
        """更新文件修改时间缓存"""
        try:
            if self.config_file.exists():
                mtime = self.config_file.stat().st_mtime
                with self._lock:
                    self._last_file_mtime = mtime
        except Exception as e:
            logger.warning(f"获取文件修改时间失败: {e}", exc_info=True)

    def start_file_watcher(self, interval: float = 2.0) -> None:
        """启动配置文件监听（后台守护线程，检测文件变化自动重载）"""
        with self._lock:
            if self._file_watcher_running:
                logger.debug("文件监听器已在运行")
                return

            self._file_watcher_interval = interval
            self._file_watcher_running = True
            self._file_watcher_stop_event.clear()

        try:
            if self.config_file.exists():
                current_mtime = self.config_file.stat().st_mtime
                if self._last_file_mtime and current_mtime > self._last_file_mtime:
                    logger.info("启动监听器时发现配置文件已变化，先执行一次重新加载")
                    with self._lock:
                        self._last_file_mtime = current_mtime
                    self.reload()
                    self._trigger_config_change_callbacks()
                elif self._last_file_mtime == 0:
                    with self._lock:
                        self._last_file_mtime = current_mtime
        except Exception as e:
            logger.warning(f"启动监听器时同步配置文件状态失败: {e}", exc_info=True)

        thread = threading.Thread(
            target=self._file_watcher_loop,
            name="ConfigFileWatcher",
            daemon=True,
        )
        with self._lock:
            self._file_watcher_thread = thread
        thread.start()
        logger.info(f"配置文件监听器已启动，检查间隔: {interval} 秒")

    def stop_file_watcher(self) -> None:
        """停止配置文件监听"""
        thread: threading.Thread | None
        with self._lock:
            if not self._file_watcher_running:
                logger.debug("文件监听器未运行")
                return

            self._file_watcher_running = False
            self._file_watcher_stop_event.set()
            thread = self._file_watcher_thread
            self._file_watcher_thread = None

        if thread:
            thread.join(timeout=1.0)
        logger.info("配置文件监听器已停止")

    def shutdown(self) -> None:
        """关闭配置管理器：刷新待保存变更、停止文件监听、取消定时器（幂等）"""
        # 先强制保存未落盘的配置变更，避免进程退出丢失数据
        try:
            if hasattr(self, "force_save") and callable(self.force_save):
                self.force_save()
        except Exception as e:
            logger.debug(f"shutdown 时强制保存失败（忽略）: {e}")

        try:
            self.stop_file_watcher()
        except Exception as e:
            logger.debug(f"关闭文件监听器失败（忽略）: {e}")

        try:
            with self._lock:
                if self._save_timer is not None:
                    self._save_timer.cancel()
                    self._save_timer = None
        except Exception as e:
            logger.debug(f"取消延迟保存定时器失败（忽略）: {e}")

    def _file_watcher_loop(self):
        """文件监听循环（后台线程主循环）"""
        logger.debug("文件监听循环已启动")
        while self._file_watcher_running:
            try:
                if self.config_file.exists():
                    current_mtime = self.config_file.stat().st_mtime
                    if current_mtime > self._last_file_mtime:
                        logger.info("检测到配置文件变化，自动重新加载")
                        with self._lock:
                            self._last_file_mtime = current_mtime
                        self.reload()
                        self._trigger_config_change_callbacks()
            except Exception as e:
                logger.warning(f"文件监听检查失败: {e}", exc_info=True)

            if self._file_watcher_stop_event.wait(self._file_watcher_interval):
                break

    def register_config_change_callback(self, callback: Callable[[], None]) -> None:
        """注册配置变更回调函数"""
        with self._lock:
            if callback not in self._config_change_callbacks:
                self._config_change_callbacks.append(callback)
                cb_name = getattr(callback, "__name__", None) or repr(callback)
                logger.debug(f"已注册配置变更回调: {cb_name}")

    def unregister_config_change_callback(self, callback: Callable[[], None]) -> None:
        """取消注册配置变更回调函数"""
        with self._lock:
            if callback in self._config_change_callbacks:
                self._config_change_callbacks.remove(callback)
                cb_name = getattr(callback, "__name__", None) or repr(callback)
                logger.debug(f"已取消配置变更回调: {cb_name}")

    def _trigger_config_change_callbacks(self):
        """触发所有配置变更回调"""
        with self._lock:
            callbacks = list(self._config_change_callbacks)

        for callback in callbacks:
            try:
                callback()
            except Exception as e:
                cb_name = getattr(callback, "__name__", None) or repr(callback)
                logger.error(f"配置变更回调执行失败 ({cb_name}): {e}", exc_info=True)

    @property
    def is_file_watcher_running(self) -> bool:
        """检查文件监听器是否在运行"""
        return self._file_watcher_running



"""后端轻量 i18n 模块：根据请求语言返回本地化字符串。

使用方式：
    from i18n import get_locale_message, detect_request_lang

    lang = detect_request_lang()  # 从 Flask request 自动检测
    msg = get_locale_message("feedback.submitted", lang)
"""

from __future__ import annotations

import logging

logger = logging.getLogger(__name__)

SUPPORTED_LANGS = ("en", "zh-CN")
DEFAULT_LANG = "en"

_MESSAGES: dict[str, dict[str, str]] = {
    "en": {
        "feedback.submitted": "Feedback submitted",
        "feedback.bodyMustBeJson": "Request body must be JSON (object)",
        "feedback.bodyMustBeObject": "Request body must be JSON object",
        "feedback.missingPrompt": "Missing field: prompt",
        "feedback.promptMustBeString": "Field prompt must be a string",
        "feedback.optionsMustBeArray": "Field predefined_options must be an array",
        "feedback.timeoutMustBeInt": "Field auto_resubmit_timeout must be an integer (seconds)",
        "feedback.contentUpdated": "Content updated",
        "feedback.serverError": "Internal server error",
        "server.shuttingDown": "Server is shutting down",
        "notify.testSuccess": "Bark test notification sent! Please check your device",
        "notify.deviceKeyEmpty": "Device Key cannot be empty",
        "notify.sendFailed": "Bark notification send failed",
        "notify.sendFailedDetail": "Bark notification send failed: {detail}",
        "notify.sendFailedConfig": "Bark notification send failed, please check configuration",
        "notify.systemUnavailable": "Notification system unavailable",
        "notify.testFailed": "Test failed",
        "notify.countZero": "count=0, ignored",
        "notify.systemDegraded": "Notification system unavailable, degraded",
        "notify.globalOff": "Notification master switch is off, ignored",
        "notify.barkDisabled": "Bark is disabled, ignored",
        "notify.barkKeyEmpty": "bark_device_key is empty, ignored",
        "notify.notTriggered": "Notification not triggered (may be disabled)",
        "notify.triggerFailed": "Trigger failed",
        "notify.configUpdated": "Notification config updated",
        "notify.noUpdateFields": "No notification config fields to update detected",
        "notify.configUnavailable": "Config system unavailable",
        "notify.updateFailed": "Update failed",
        "notify.getFailed": "Failed to get config",
    },
    "zh-CN": {
        "feedback.submitted": "反馈已提交",
        "feedback.bodyMustBeJson": "请求体必须是 JSON（object）",
        "feedback.bodyMustBeObject": "请求体必须是 JSON object",
        "feedback.missingPrompt": "缺少字段：prompt",
        "feedback.promptMustBeString": "字段 prompt 必须是字符串",
        "feedback.optionsMustBeArray": "字段 predefined_options 必须是数组",
        "feedback.timeoutMustBeInt": "字段 auto_resubmit_timeout 必须是整数（秒）",
        "feedback.contentUpdated": "内容已更新",
        "feedback.serverError": "服务器内部错误",
        "server.shuttingDown": "服务即将关闭",
        "notify.testSuccess": "Bark 测试通知发送成功！请检查设备",
        "notify.deviceKeyEmpty": "Device Key 不能为空",
        "notify.sendFailed": "Bark 通知发送失败",
        "notify.sendFailedDetail": "Bark 通知发送失败：{detail}",
        "notify.sendFailedConfig": "Bark 通知发送失败，请检查配置",
        "notify.systemUnavailable": "通知系统不可用",
        "notify.testFailed": "测试失败",
        "notify.countZero": "count=0，已忽略",
        "notify.systemDegraded": "通知系统不可用，已降级",
        "notify.globalOff": "通知总开关未开启，已忽略",
        "notify.barkDisabled": "Bark 未启用，已忽略",
        "notify.barkKeyEmpty": "bark_device_key 为空，已忽略",
        "notify.notTriggered": "通知未触发（可能已禁用）",
        "notify.triggerFailed": "触发失败",
        "notify.configUpdated": "通知配置已更新",
        "notify.noUpdateFields": "未检测到需要更新的通知配置字段",
        "notify.configUnavailable": "配置系统不可用",
        "notify.updateFailed": "更新失败",
        "notify.getFailed": "获取配置失败",
    },
}


def normalize_lang(raw: str) -> str:
    """将语言标识归一化为支持的语言代码。"""
    s = (raw or "").strip().lower()
    if s.startswith("zh"):
        return "zh-CN"
    if s.startswith("en"):
        return "en"
    return DEFAULT_LANG


def detect_request_lang() -> str:
    """从 Flask request 的 Accept-Language 头检测语言，回退到 config.toml 配置。"""
    try:
        from flask import request

        accept = request.headers.get("Accept-Language", "")
        if accept:
            primary = accept.split(",")[0].split(";")[0].strip()
            lang = normalize_lang(primary)
            if lang in SUPPORTED_LANGS:
                return lang
    except (RuntimeError, ImportError):
        pass

    try:
        from config_manager import get_config

        ui_lang = get_config().get_section("web_ui").get("language", "auto")
        if ui_lang != "auto":
            return normalize_lang(ui_lang)
    except Exception:
        pass

    return DEFAULT_LANG


def get_locale_message(
    key: str,
    lang: str | None = None,
    **kwargs: str,
) -> str:
    """获取本地化消息字符串。

    Args:
        key: 消息键（如 "feedback.submitted"）
        lang: 语言代码（None 则自动检测）
        **kwargs: 模板参数（如 detail="xxx"）
    """
    if lang is None:
        lang = detect_request_lang()

    messages = _MESSAGES.get(lang, _MESSAGES[DEFAULT_LANG])
    val = messages.get(key)

    if val is None:
        val = _MESSAGES[DEFAULT_LANG].get(key, key)

    if kwargs:
        try:
            val = val.format(**kwargs)
        except (KeyError, IndexError):
            pass

    return val


# 便捷别名
msg = get_locale_message



"""上传文件处理工具 — 从 Flask request 提取已验证的图片列表。

供 task.py 和 feedback.py 中的提交端点共用，消除图片处理逻辑重复。
"""

from __future__ import annotations

import base64
import hashlib
import uuid
from pathlib import Path
from typing import Any

from flask import Request

from enhanced_logging import EnhancedLogger
from file_validator import validate_uploaded_file

logger = EnhancedLogger(__name__)


def extract_uploaded_images(
    request: Request, *, include_metadata: bool = False
) -> list[dict[str, Any]]:
    """从 Flask request.files 中提取所有 image_* 文件，返回验证通过的图片列表。

    参数:
        request: Flask 请求对象
        include_metadata: 是否包含额外元数据（safe_filename, file_hash, validation_warnings）

    返回:
        已验证的图片字典列表，每个包含 filename, data(base64), content_type, size
    """
    images: list[dict[str, Any]] = []
    for key in request.files:
        if not key.startswith("image_"):
            continue
        file = request.files[key]
        if not file or not file.filename:
            continue
        try:
            file_content = file.read()
            validation_result = validate_uploaded_file(
                file_content, file.filename, file.content_type
            )
            if not validation_result["valid"]:
                logger.warning(
                    f"文件验证失败: {file.filename} - {'; '.join(validation_result['errors'])}"
                )
                continue

            if validation_result.get("warnings"):
                logger.info(
                    f"文件验证警告: {file.filename} - {'; '.join(validation_result['warnings'])}"
                )

            original_filename = Path(file.filename.replace("\\", "/")).name
            base64_data = base64.b64encode(file_content).decode("utf-8")
            mime = validation_result["mime_type"] or file.content_type or "image/jpeg"

            entry: dict[str, Any] = {
                "filename": original_filename,
                "data": base64_data,
                "content_type": mime,
                "size": len(file_content),
            }

            if include_metadata:
                entry["safe_filename"] = (
                    f"{uuid.uuid4().hex}{validation_result.get('extension', '.bin')}"
                )
                entry["file_hash"] = hashlib.sha256(file_content).hexdigest()[:16]
                entry["validated_type"] = validation_result["file_type"]
                entry["validation_warnings"] = validation_result["warnings"]

            images.append(entry)
            logger.debug(
                f"处理图片: {original_filename} ({len(file_content)} bytes) "
                f"- 类型: {validation_result['file_type']}"
            )
        except Exception as e:
            logger.error(f"处理文件 {file.filename} 时出错: {e}", exc_info=True)
    return images



"""反馈提交路由 Mixin — 通用提交、内容更新、反馈查询。"""

from __future__ import annotations

import json
from typing import TYPE_CHECKING, Any, cast

from flask import jsonify, request
from flask.typing import ResponseReturnValue

from enhanced_logging import EnhancedLogger
from i18n import msg
from server import get_task_queue
from shared_types import FeedbackResult
from web_ui_routes._upload_helpers import extract_uploaded_images

if TYPE_CHECKING:
    import threading

    from flask import Flask
    from flask_limiter import Limiter

logger = EnhancedLogger(__name__)


class FeedbackRoutesMixin:
    """提供 3 个反馈相关 API 路由，由 WebFeedbackUI 通过 MRO 继承。"""

    if TYPE_CHECKING:
        app: Flask
        limiter: Limiter
        _state_lock: threading.RLock
        feedback_result: FeedbackResult | None
        current_prompt: str
        current_options: list[str]
        current_task_id: str | None
        current_auto_resubmit_timeout: int
        _single_task_timeout_explicit: bool
        has_content: bool

        def render_markdown(self, text: str) -> str: ...

    def _setup_feedback_routes(self) -> None:
        from web_ui import (
            _get_default_auto_resubmit_timeout_from_config,
            validate_auto_resubmit_timeout,
        )

        @self.app.route("/api/submit", methods=["POST"])
        @self.limiter.limit("60 per minute")
        def submit_feedback() -> ResponseReturnValue:
            """提交用户反馈（文本 / 选项 / 图片）
            ---
            tags:
              - Feedback
            consumes:
              - multipart/form-data
              - application/json
            parameters:
              - in: formData
                name: task_id
                type: string
                description: 目标任务 ID
              - in: formData
                name: feedback_text
                type: string
                description: 反馈文本
              - in: formData
                name: selected_options
                type: string
                description: 已选选项（JSON 数组字符串）
              - in: formData
                name: images
                type: file
                description: 上传的图片文件
            responses:
              200:
                description: 提交成功
                schema:
                  type: object
                  properties:
                    status:
                      type: string
                      example: success
                    message:
                      type: string
                    persistent:
                      type: boolean
                    clear_content:
                      type: boolean
            """
            logger.info(f"收到提交请求 - Content-Type: {request.content_type}")
            logger.debug(f"request.files keys: {list(request.files.keys())}")
            logger.debug(f"request.form keys: {list(request.form.keys())}")
            try:
                json_data = request.get_json()
                logger.debug(
                    f"request.json keys: {list(json_data.keys()) if isinstance(json_data, dict) else type(json_data)}"
                )
            except Exception as e:
                logger.debug(f"无法解析JSON数据: {e}")

            if request.files:
                requested_task_id = request.form.get("task_id", "").strip()
                feedback_text = request.form.get("feedback_text", "").strip()
                selected_options_str = request.form.get("selected_options", "[]")
                try:
                    selected_options = json.loads(selected_options_str)
                except json.JSONDecodeError:
                    selected_options = []

                logger.debug("接收到的反馈数据:")
                logger.debug(f"  - 文字内容长度: {len(feedback_text)}")
                logger.debug(f"  - 选项数量: {len(selected_options)}")
                logger.debug(f"  - 文件数量: {len(request.files)}")

                images: list[Any] = extract_uploaded_images(
                    request, include_metadata=True
                )
            elif request.form:
                requested_task_id = request.form.get("task_id", "").strip()
                feedback_text = request.form.get("feedback_text", "").strip()
                selected_options_str = request.form.get("selected_options", "[]")
                try:
                    selected_options = json.loads(selected_options_str)
                except json.JSONDecodeError:
                    selected_options = []

                logger.debug("接收到的表单数据:")
                logger.debug(
                    f"  - 文字内容: '{feedback_text}' (长度: {len(feedback_text)})"
                )
                logger.debug(f"  - 选项数据: {selected_options_str}")
                logger.debug(f"  - 解析后选项: {selected_options}")

                images = []
            else:
                try:
                    data = request.get_json() or {}
                    requested_task_id = str(data.get("task_id", "")).strip()
                    feedback_text = str(
                        data.get("feedback_text", data.get("user_input", ""))
                    ).strip()
                    selected_options = data.get("selected_options", [])
                    images = data.get("images", [])

                    logger.debug("接收到的JSON数据:")
                    logger.debug(
                        f"  - 文字内容: '{feedback_text}' (长度: {len(feedback_text)})"
                    )
                    logger.debug(f"  - 选项: {selected_options}")
                    logger.debug(f"  - 图片数量: {len(images)}")
                except Exception:
                    requested_task_id = ""
                    feedback_text = ""
                    selected_options = []
                    images = []
                    logger.debug("JSON解析失败，使用默认值")

            task_queue = get_task_queue()
            target_task_id = requested_task_id
            if requested_task_id:
                target_task = task_queue.get_task(requested_task_id)
                if not target_task:
                    logger.warning(f"提交反馈时任务不存在: {requested_task_id}")
                    return (
                        jsonify({"success": False, "error": "任务不存在"}),
                        404,
                    )
            else:
                active_task = task_queue.get_active_task()
                target_task_id = active_task.task_id if active_task else ""

            feedback_result: FeedbackResult = {
                "user_input": feedback_text,
                "selected_options": selected_options,
                "images": images,  # type: ignore[typeddict-item]  # 运行时总是正确类型
            }

            with self._state_lock:
                self.feedback_result = feedback_result

            logger.debug("最终存储的反馈结果:")
            logger.debug(
                f"  - user_input: '{feedback_result['user_input']}' (长度: {len(feedback_result['user_input'])})"
            )
            logger.debug(f"  - selected_options: {feedback_result['selected_options']}")
            logger.debug(f"  - images数量: {len(feedback_result['images'])}")

            if target_task_id:
                logger.info(f"同时将反馈提交到TaskQueue中的目标任务: {target_task_id}")
                task_queue.complete_task(
                    target_task_id,
                    cast(dict[str, Any], feedback_result),
                )

            with self._state_lock:
                self.current_prompt = ""
                self.current_options = []
                self.has_content = False
            return jsonify(
                {
                    "status": "success",
                    "message": msg("feedback.submitted"),
                    "persistent": True,
                    "clear_content": True,
                }
            )

        @self.app.route("/api/update", methods=["POST"])
        def update_content() -> ResponseReturnValue:
            """更新页面内容（单任务模式）
            ---
            tags:
              - Feedback
            consumes:
              - application/json
            parameters:
              - in: body
                name: body
                required: true
                schema:
                  type: object
                  properties:
                    prompt:
                      type: string
                      description: 新的提示文本（Markdown 格式）
                    predefined_options:
                      type: array
                      items:
                        type: string
                    task_id:
                      type: string
                    auto_resubmit_timeout:
                      type: number
                      description: 超时时间（秒，最大 250）
            responses:
              200:
                description: 更新成功
                schema:
                  type: object
                  properties:
                    status:
                      type: string
                      example: success
                    message:
                      type: string
                    prompt:
                      type: string
                    prompt_html:
                      type: string
                      description: Markdown 渲染后的 HTML
                    predefined_options:
                      type: array
                      items:
                        type: string
                    task_id:
                      type: string
                    auto_resubmit_timeout:
                      type: number
                    has_content:
                      type: boolean
              400:
                description: 请求参数错误
              500:
                description: 服务器内部错误
            """
            try:
                raw = request.get_json(silent=False)
            except Exception:
                return (
                    jsonify(
                        {
                            "status": "error",
                            "error": "invalid_json",
                            "message": msg("feedback.bodyMustBeJson"),
                        }
                    ),
                    400,
                )

            if not isinstance(raw, dict):
                return (
                    jsonify(
                        {
                            "status": "error",
                            "error": "invalid_body",
                            "message": msg("feedback.bodyMustBeObject"),
                        }
                    ),
                    400,
                )

            data: dict[str, Any] = raw

            if "prompt" not in data:
                return (
                    jsonify(
                        {
                            "status": "error",
                            "error": "missing_field",
                            "message": msg("feedback.missingPrompt"),
                        }
                    ),
                    400,
                )
            new_prompt_raw = data.get("prompt")
            if not isinstance(new_prompt_raw, str):
                return (
                    jsonify(
                        {
                            "status": "error",
                            "error": "invalid_field_type",
                            "message": msg("feedback.promptMustBeString"),
                        }
                    ),
                    400,
                )
            new_prompt = new_prompt_raw

            if len(new_prompt) > 10000:
                logger.warning(
                    f"/api/update prompt 过长：{len(new_prompt)}，将截断到 10000"
                )
                new_prompt = new_prompt[:10000] + "..."

            new_options_raw = data.get("predefined_options", [])
            if new_options_raw is None:
                new_options_raw = []
            if not isinstance(new_options_raw, list):
                return (
                    jsonify(
                        {
                            "status": "error",
                            "error": "invalid_field_type",
                            "message": msg("feedback.optionsMustBeArray"),
                        }
                    ),
                    400,
                )

            new_options: list[str] = []
            for opt in new_options_raw:
                if not isinstance(opt, str):
                    continue
                t = opt.strip()
                if not t:
                    continue
                if len(t) > 500:
                    new_options.append(t[:500] + "...")
                else:
                    new_options.append(t)

            new_task_id_raw = data.get("task_id")
            if new_task_id_raw is None:
                new_task_id = None
            elif isinstance(new_task_id_raw, str):
                t = new_task_id_raw.strip()
                new_task_id = t if t else None
            else:
                t = str(new_task_id_raw).strip()
                new_task_id = t if t else None

            default_timeout = _get_default_auto_resubmit_timeout_from_config()
            timeout_explicit = "auto_resubmit_timeout" in data
            if timeout_explicit:
                raw_timeout = data.get("auto_resubmit_timeout")
                try:
                    timeout_int = int(raw_timeout)
                except Exception:
                    return (
                        jsonify(
                            {
                                "status": "error",
                                "error": "invalid_field_value",
                                "message": msg("feedback.timeoutMustBeInt"),
                            }
                        ),
                        400,
                    )
            else:
                timeout_int = default_timeout

            new_auto_resubmit_timeout = validate_auto_resubmit_timeout(timeout_int)

            try:
                with self._state_lock:
                    self.current_prompt = new_prompt
                    self.current_options = new_options
                    self.current_task_id = new_task_id
                    self.current_auto_resubmit_timeout = new_auto_resubmit_timeout
                    self._single_task_timeout_explicit = timeout_explicit
                    self.has_content = bool(new_prompt.strip())
                    self.feedback_result = None

                    prompt_snapshot = str(self.current_prompt)
                    options_snapshot = list(self.current_options)
                    task_id_snapshot = self.current_task_id
                    timeout_snapshot = int(self.current_auto_resubmit_timeout)
                    has_content_snapshot = bool(self.has_content)

                prompt_html = ""
                if has_content_snapshot:
                    try:
                        prompt_html = self.render_markdown(prompt_snapshot)
                    except Exception as e:
                        logger.warning(
                            f"/api/update prompt 渲染失败: {e}", exc_info=True
                        )
                        prompt_html = ""

                return jsonify(
                    {
                        "status": "success",
                        "message": msg("feedback.contentUpdated"),
                        "prompt": prompt_snapshot,
                        "prompt_html": prompt_html,
                        "predefined_options": options_snapshot,
                        "task_id": task_id_snapshot,
                        "auto_resubmit_timeout": timeout_snapshot,
                        "has_content": has_content_snapshot,
                    }
                )
            except Exception as e:
                logger.error(f"/api/update 处理失败: {e}", exc_info=True)
                return (
                    jsonify(
                        {
                            "status": "error",
                            "error": "internal_error",
                            "message": msg("feedback.serverError"),
                        }
                    ),
                    500,
                )

        @self.app.route("/api/feedback", methods=["GET"])
        def get_feedback() -> ResponseReturnValue:
            """获取用户反馈结果（单任务模式）
            ---
            tags:
              - Feedback
            responses:
              200:
                description: 反馈结果（读后清除）
                schema:
                  type: object
                  properties:
                    status:
                      type: string
                      enum: [success, waiting]
                    feedback:
                      type: object
                      description: 反馈内容，无反馈时为 null
            """
            with self._state_lock:
                result = self.feedback_result
                self.feedback_result = None
            if result:
                return jsonify({"status": "success", "feedback": result})
            return jsonify({"status": "waiting", "feedback": None})



#!/usr/bin/env python3
"""
配置管理模块：TOML 配置文件的跨平台加载、读写、热重载。

核心特性：使用可重入锁（RLock）保护共享状态、延迟保存优化、network_security 独立管理、文件变更监听。
旧 JSONC/JSON 文件在首次加载时自动迁移为 TOML。
通过 get_config() 获取全局 ConfigManager 实例。
"""

import json
import logging
import os
import platform
import re
import shutil
import sys
import tempfile
import threading
import time
from collections.abc import Callable, Generator
from contextlib import contextmanager
from pathlib import Path
from typing import Any, cast

import tomlkit

from exceptions import ConfigValidationError
from shared_types import SECTION_MODELS

try:
    from platformdirs import user_config_dir

    PLATFORMDIRS_AVAILABLE = True
except ImportError:
    PLATFORMDIRS_AVAILABLE = False

from config_modules import (
    FileWatcherMixin,
    IOOperationsMixin,
    NetworkSecurityMixin,
    TomlEngineMixin,
)

logger = logging.getLogger(__name__)

# =========================
# 日志脱敏（避免泄露密钥/Token）
# =========================


def _is_sensitive_config_key(key: str) -> bool:
    lowered = (key or "").lower()
    # 只做最小必要的脱敏：Bark device_key 等敏感标识一律不应出现在日志
    return any(
        token in lowered
        for token in (
            "device_key",
            "devicekey",
            "token",
            "secret",
            "password",
            "passwd",
            "api_key",
            "apikey",
            "private_key",
        )
    )


def _sanitize_config_value_for_log(key: str, value: Any) -> str:
    if _is_sensitive_config_key(key):
        return "<redacted>"
    try:
        text = str(value)
    except Exception:
        return "<unprintable>"
    # 避免日志过长
    return text if len(text) <= 200 else (text[:200] + "...")


class ReadWriteLock:
    """
    读写锁：多读者并发、写者独占，基于 Condition + RLock 实现。

    注意：本类目前未被 ConfigManager 使用（ConfigManager 使用 threading.RLock），
    作为独立工具类保留，供需要读写分离锁场景的调用方使用。
    """

    def __init__(self):
        """初始化读写锁"""
        self._read_ready = threading.Condition(threading.RLock())
        self._readers = 0

    @contextmanager
    def read_lock(self) -> Generator[None, None, None]:
        """获取读锁（多读者并发，仅在写者持有锁时阻塞）"""
        self._read_ready.acquire()
        try:
            self._readers += 1
        finally:
            self._read_ready.release()

        try:
            yield
        finally:
            self._read_ready.acquire()
            try:
                self._readers -= 1
                if self._readers == 0:
                    self._read_ready.notify_all()
            finally:
                self._read_ready.release()

    @contextmanager
    def write_lock(self) -> Generator[None, None, None]:
        """获取写锁（独占访问，等待所有读者退出）"""
        self._read_ready.acquire()
        try:
            while self._readers > 0:
                self._read_ready.wait()
            yield
        finally:
            self._read_ready.release()


def parse_jsonc(content: str) -> dict[str, Any]:
    """
    解析 JSONC（带注释的 JSON）为字典，支持 // 单行注释和 /* */ 多行注释。

    异常:
        json.JSONDecodeError: JSON 语法错误时抛出
    """
    cleaned_chars = []
    in_string = False
    escape_next = False
    in_single_line_comment = False
    in_multi_line_comment = False

    i = 0
    while i < len(content):
        char = content[i]
        next_char = content[i + 1] if i + 1 < len(content) else ""

        if in_single_line_comment:
            if char == "\n":
                in_single_line_comment = False
                cleaned_chars.append(char)
            i += 1
            continue

        if in_multi_line_comment:
            if char == "*" and next_char == "/":
                in_multi_line_comment = False
                i += 2
            else:
                i += 1
            continue

        if in_string:
            cleaned_chars.append(char)
            if escape_next:
                escape_next = False
            elif char == "\\":
                escape_next = True
            elif char == '"':
                in_string = False
            i += 1
            continue

        if char == '"':
            in_string = True
            cleaned_chars.append(char)
            i += 1
            continue

        if char == "/" and next_char == "/":
            in_single_line_comment = True
            i += 2
            continue

        if char == "/" and next_char == "*":
            in_multi_line_comment = True
            i += 2
            continue

        cleaned_chars.append(char)
        i += 1

    cleaned_content = "".join(cleaned_chars)

    # JSONC 允许尾部逗号，但 json.loads 不接受，需预处理移除
    cleaned_content = re.sub(r",\s*([}\]])", r"\1", cleaned_content)

    return cast(dict[str, Any], json.loads(cleaned_content))


def _is_uvx_mode() -> bool:
    """
    检测是否应使用“用户配置目录”（uvx/安装模式）而非“开发模式”。

    说明
    ----
    - **用户模式（True）**：使用用户配置目录（跨平台标准路径）。
      - uvx 运行（推荐给普通用户）
      - 通过 pip/uv 安装后运行（避免在任意项目目录意外生成 config.jsonc）
    - **开发模式（False）**：优先使用当前目录配置（从仓库运行时更方便调试）。

    判定规则
    ----
    1) 若检测到 uvx 运行特征（sys.executable 含 uvx 或 UVX_PROJECT 环境变量），返回 True
    2) 若当前代码看起来位于本仓库源码树内，且当前工作目录位于该源码树内，返回 False
    3) 其他情况（默认）：返回 True
    """
    executable_path = sys.executable
    if "uvx" in executable_path or ".local/share/uvx" in executable_path:
        return True

    # 检查环境变量
    if os.getenv("UVX_PROJECT"):
        return True

    # 仅当“代码本身位于仓库源码树”且 cwd 位于该源码树内时，才视为开发模式。
    # 避免在普通用户的任意 git 仓库/项目目录中误判为开发模式，导致配置文件被创建在当前目录。
    try:
        module_dir = Path(__file__).resolve().parent
        is_repo_checkout = (module_dir / "pyproject.toml").exists() and (
            module_dir / "server.py"
        ).exists()
        if is_repo_checkout:
            cwd = Path.cwd().resolve()
            if cwd == module_dir or module_dir in cwd.parents:
                return False
    except Exception:
        # 任何判定异常都降级为用户模式（更安全/更符合文档预期）
        pass

    return True


def find_config_file(config_filename: str = "config.toml") -> Path:
    """
    查找配置文件路径，支持环境变量覆盖、uvx 模式和开发模式。

    查找优先级（开发模式）：当前目录 > 用户配置目录 > 创建新配置。
    格式优先级：TOML > JSONC > JSON（向后兼容）。
    uvx 模式仅使用用户配置目录。
    跨平台配置目录：Linux ~/.config、macOS ~/Library/Application Support、Windows %APPDATA%。
    """
    requested_path = Path(config_filename).expanduser()
    if requested_path.is_absolute() or requested_path.parent != Path("."):
        return requested_path

    override = os.environ.get("AI_INTERVENTION_AGENT_CONFIG_FILE")
    if override:
        override_path = Path(override).expanduser()
        if override_path.is_dir() or override.endswith(("/", "\\")):
            override_path = override_path / config_filename
        logger.info(
            f"使用环境变量 AI_INTERVENTION_AGENT_CONFIG_FILE 指定配置文件: {override_path}"
        )
        return override_path

    is_uvx_mode = _is_uvx_mode()

    if is_uvx_mode:
        logger.info("检测到用户模式（uvx/安装），使用用户配置目录")
    else:
        logger.info("检测到开发模式，优先使用当前目录配置")

    # 向后兼容的候选文件名列表（TOML 优先）
    _COMPAT_NAMES = ("config.toml", "config.jsonc", "config.json")

    if not is_uvx_mode:
        # 开发模式：检查当前工作目录
        for name in _COMPAT_NAMES:
            candidate = Path(name)
            if candidate.exists():
                logger.info(f"使用当前目录的配置文件: {candidate.absolute()}")
                return candidate

    try:
        try:
            if not PLATFORMDIRS_AVAILABLE:
                raise ImportError("platformdirs not available")
            user_config_dir_path = Path(user_config_dir("ai-intervention-agent"))
        except ImportError:
            user_config_dir_path = _get_user_config_dir_fallback()

        # 按优先级搜索用户配置目录
        for name in _COMPAT_NAMES:
            candidate = user_config_dir_path / name
            if candidate.exists():
                logger.info(f"使用用户配置目录的配置文件: {candidate}")
                return candidate

        # 都不存在，返回 TOML 路径（用于创建默认配置）
        user_config_file = user_config_dir_path / config_filename
        logger.info(f"配置文件不存在，将在用户配置目录创建: {user_config_file}")
        return user_config_file

    except Exception as e:
        logger.warning(f"获取用户配置目录失败: {e}，使用当前目录", exc_info=True)
        return Path(config_filename)


def _get_user_config_dir_fallback() -> Path:
    """
    platformdirs 不可用时的回退实现，返回跨平台标准配置目录。

    Windows: %APPDATA%、macOS: ~/Library/Application Support、Linux: $XDG_CONFIG_HOME 或 ~/.config。
    """
    system = platform.system().lower()
    home = Path.home()

    if system == "windows":
        appdata = os.getenv("APPDATA")
        if appdata:
            return Path(appdata) / "ai-intervention-agent"
        else:
            return home / "AppData" / "Roaming" / "ai-intervention-agent"
    elif system == "darwin":
        return home / "Library" / "Application Support" / "ai-intervention-agent"
    else:
        xdg_config_home = os.getenv("XDG_CONFIG_HOME")
        if xdg_config_home:
            return Path(xdg_config_home) / "ai-intervention-agent"
        else:
            return home / ".config" / "ai-intervention-agent"


class ConfigManager(
    TomlEngineMixin,
    NetworkSecurityMixin,
    FileWatcherMixin,
    IOOperationsMixin,
):
    """
    配置管理器：TOML 配置文件的加载、读写、持久化、热重载。

    核心特性：使用可重入锁（RLock）保护共享状态、延迟保存优化、network_security 独立管理（带缓存）、
    文件变更监听、配置导入导出。通过模块级 config_manager 全局实例访问。

    支持格式：TOML（主格式）。旧 JSONC/JSON 文件在首次加载时自动迁移为 TOML。

    路由通过 Mixin 拆分（各 Mixin 定义在 config_modules/ 下）：
    - TomlEngineMixin: TOML 格式解析/保存（保留注释）
    - NetworkSecurityMixin: network_security 段校验/读写
    - FileWatcherMixin: 文件监听/回调/shutdown
    - IOOperationsMixin: 配置导出/导入/备份/恢复
    """

    def __init__(self, config_file: str = "config.toml"):
        """初始化配置管理器：查找配置文件、初始化锁和缓存、加载配置、启动文件监听"""
        # 判断是否为显式路径（绝对/含目录层级）——仅自动发现的旧文件才做 JSONC→TOML 迁移
        req = Path(config_file).expanduser()
        self._explicit_path = req.is_absolute() or req.parent != Path(".")
        self.config_file = find_config_file(config_file)

        # 初始化配置字典
        self._config: dict[str, Any] = {}

        # 初始化锁机制
        self._lock = threading.RLock()  # 可重入锁，用于延迟保存定时器

        # 初始化文件内容和访问时间
        self._original_content: str | None = None  # 保存原始文件内容（用于保留注释）
        self._last_access_time = time.monotonic()  # 跟踪最后访问时间

        # 性能优化：配置写入缓冲机制
        self._pending_changes: dict[str, Any] = {}  # 待写入的配置变更
        self._save_timer: threading.Timer | None = None  # 延迟保存定时器
        self._save_delay = 3.0  # 延迟保存时间（秒）
        self._last_save_time: float = 0  # 上次保存时间（monotonic）

        # 【性能优化】network_security 配置缓存
        self._network_security_cache: dict[str, Any] | None = None
        self._network_security_cache_time: float = 0  # monotonic
        self._network_security_cache_ttl: float = 30.0  # 30 秒缓存有效期

        # 【性能优化】通用 section 缓存层
        self._section_cache: dict[str, dict[str, Any]] = {}
        self._section_cache_time: dict[str, float] = {}
        self._section_cache_ttl: float = 10.0  # section 缓存有效期（秒）

        # 【性能优化】缓存统计
        self._cache_stats = {
            "hits": 0,  # 缓存命中次数
            "misses": 0,  # 缓存未命中次数
            "invalidations": 0,  # 缓存失效次数
        }

        # 【新增】文件监听相关属性
        self._file_watcher_thread: threading.Thread | None = None
        self._file_watcher_running = False
        self._file_watcher_stop_event = threading.Event()  # 用于优雅停止
        self._file_watcher_interval = 2.0  # 检查间隔（秒）
        self._last_file_mtime: float = 0  # 上次文件修改时间
        self._config_change_callbacks: list[
            Callable[[], None]
        ] = []  # 配置变更回调函数列表

        # 加载配置文件
        self._load_config()

        # 初始化文件修改时间
        self._update_file_mtime()

    def _get_default_config(self) -> dict[str, Any]:
        """返回默认配置字典（由 Pydantic 段模型的默认值生成，单一真相源）"""
        return {name: model().model_dump() for name, model in SECTION_MODELS.items()}

    @staticmethod
    def _exclude_network_security(config: dict[str, Any]) -> dict[str, Any]:
        """从配置字典中排除 network_security（返回新字典或原地修改）"""
        if "network_security" in config:
            del config["network_security"]
            logger.debug("已从配置中排除 network_security")
        return config

    def _is_toml_file(self) -> bool:
        """判断当前配置文件是否为 TOML 格式"""
        return self.config_file.suffix.lower() == ".toml"

    def _parse_config_content(self, content: str) -> dict[str, Any]:
        """根据当前文件格式解析配置内容（TOML 或降级到 JSON）"""
        if self._is_toml_file():
            return self._parse_toml(content)
        return cast(dict[str, Any], json.loads(content))

    def _migrate_jsonc_to_toml(self) -> bool:
        """将旧的 JSONC/JSON 配置文件迁移为 TOML 格式"""
        old_file = self.config_file
        new_file = old_file.with_suffix(".toml")
        try:
            with open(old_file, encoding="utf-8") as f:
                content = f.read()
            if old_file.suffix.lower() == ".jsonc":
                config_data = parse_jsonc(content)
            else:
                config_data = json.loads(content)
            mdns = config_data.get("mdns", {})
            if isinstance(mdns, dict) and mdns.get("enabled") is None:
                mdns["enabled"] = "auto"
            template_file = Path(__file__).parent / "config.toml.default"
            if template_file.exists():
                with open(template_file, encoding="utf-8") as f:
                    doc = tomlkit.parse(f.read())
                for sk, sv in config_data.items():
                    if isinstance(sv, dict) and sk in doc:
                        section = doc[sk]
                        if isinstance(section, dict):
                            for k, v in sv.items():
                                section[k] = v
                    elif sk not in doc:
                        doc[sk] = sv
                toml_content = tomlkit.dumps(doc)
            else:
                toml_content = tomlkit.dumps(tomlkit.item(config_data))
            with open(new_file, "w", encoding="utf-8") as f:
                f.write(toml_content)
            backup = old_file.with_suffix(old_file.suffix + ".bak")
            old_file.rename(backup)
            logger.info(
                f"配置已迁移: {old_file.name} -> {new_file.name} (备份: {backup.name})"
            )
            self.config_file = new_file
            return True
        except Exception as e:
            logger.error(f"JSONC->TOML 迁移失败: {e}", exc_info=True)
            return False

    def _load_config(self):
        """从磁盘加载配置文件，排除 network_security，合并默认配置"""
        with self._lock:
            # 【可靠性】加载失败时回滚到上一次成功配置，避免“编辑中间态/损坏文件”导致回退到默认值
            had_previous_config = bool(self._config)
            previous_config = self._config.copy()
            previous_original_content = self._original_content
            try:
                # 自动迁移旧 JSONC/JSON 格式（仅自动发现的文件，显式路径不迁移）
                if (
                    self.config_file.exists()
                    and not self._is_toml_file()
                    and not self._explicit_path
                ):
                    self._migrate_jsonc_to_toml()

                if self.config_file.exists():
                    with open(self.config_file, encoding="utf-8") as f:
                        content = f.read()

                    full_config = self._parse_config_content(content)
                    fmt = self.config_file.suffix.lstrip(".")
                    logger.info(f"{fmt.upper()} 配置文件已加载: {self.config_file}")

                    # 【健壮性】加载时也做结构校验（重复数组定义/类型错误），避免静默吞掉损坏配置
                    self._validate_config_structure(full_config, content)

                    # 保存原始内容（用于保留注释）——仅在解析与结构校验成功后更新
                    self._original_content = content

                    # 完全排除 network_security，不加载到内存中
                    self._config = {}
                    for key, value in full_config.items():
                        if key != "network_security":
                            self._config[key] = value

                    if "network_security" in full_config:
                        logger.debug("network_security 配置已排除，不加载到内存中")
                else:
                    # 创建默认配置文件
                    self._config = self._exclude_network_security(
                        self._get_default_config()
                    )
                    self._original_content = None
                    self._create_default_config_file()
                    logger.info(f"创建默认配置文件: {self.config_file}")

                # 合并默认配置（确保新增的配置项存在）
                default_config = self._exclude_network_security(
                    self._get_default_config()
                )
                self._config = self._merge_config(default_config, self._config)

            except Exception as e:
                logger.error(f"加载配置文件失败: {e}", exc_info=True)
                if had_previous_config:
                    self._config = previous_config
                    self._original_content = previous_original_content
                    logger.warning(
                        "加载配置失败，已保留上一次成功加载的内存配置（避免回退到默认值）"
                    )
                else:
                    self._config = self._exclude_network_security(
                        self._get_default_config()
                    )
                    self._original_content = None

    def _merge_config(
        self, default: dict[str, Any], current: dict[str, Any]
    ) -> dict[str, Any]:
        """递归合并配置：补充缺失的默认键，保持用户值优先，排除 network_security"""
        result = current.copy()  # 以当前配置为基础

        # 只添加缺失的默认键，不修改现有值
        for key, default_value in default.items():
            # 额外安全措施：确保不合并 network_security
            if key == "network_security":
                logger.debug("_merge_config: 跳过 network_security 配置")
                continue

            if key not in result:
                # 缺失的键，使用默认值
                result[key] = default_value
            elif isinstance(result[key], dict) and isinstance(default_value, dict):
                # 递归合并嵌套字典，但保持现有值优先
                result[key] = self._merge_config(default_value, result[key])

        # 确保结果中不包含 network_security
        self._exclude_network_security(result)
        return result

    def _create_default_config_file(self):
        """创建带注释的默认配置文件（使用 TOML 模板）"""
        try:
            self.config_file.parent.mkdir(parents=True, exist_ok=True)

            toml_template = Path(__file__).parent / "config.toml.default"

            if self._is_toml_file() and toml_template.exists():
                shutil.copy2(toml_template, self.config_file)
                with open(toml_template, encoding="utf-8") as f:
                    self._original_content = f.read()
                logger.info(f"已从 TOML 模板创建默认配置文件: {self.config_file}")
            else:
                logger.warning("模板文件不存在，使用默认配置创建文件")
                default_config = self._exclude_network_security(
                    self._get_default_config()
                )
                if self._is_toml_file():
                    content = tomlkit.dumps(tomlkit.item(default_config))
                else:
                    content = json.dumps(default_config, indent=2, ensure_ascii=False)
                with open(self.config_file, "w", encoding="utf-8") as f:
                    f.write(content)
                self._original_content = content
                logger.info(f"已创建默认配置文件: {self.config_file}")

        except Exception as e:
            logger.error(f"创建默认配置文件失败: {e}", exc_info=True)
            try:
                default_config = self._exclude_network_security(
                    self._get_default_config()
                )
                content = json.dumps(default_config, indent=2, ensure_ascii=False)
                with open(self.config_file, "w", encoding="utf-8") as f:
                    f.write(content)
                self._original_content = content
                logger.info(f"回退创建 JSON 配置文件成功: {self.config_file}")
            except Exception as fallback_error:
                logger.error(f"回退创建配置文件也失败: {fallback_error}", exc_info=True)
                raise

    def _schedule_save(self):
        """调度延迟保存（默认3秒后执行，多次调用合并为一次保存）"""
        with self._lock:
            # 取消之前的保存定时器
            if self._save_timer is not None:
                self._save_timer.cancel()

            # 设置新的延迟保存定时器
            self._save_timer = threading.Timer(self._save_delay, self._delayed_save)
            # 【可靠性】Timer 默认非守护线程，可能导致测试/进程退出被阻塞
            self._save_timer.daemon = True
            self._save_timer.start()
            logger.debug(f"已调度配置保存，将在 {self._save_delay} 秒后执行")

    def _delayed_save(self):
        """延迟保存定时器回调：应用待保存变更并写入文件"""
        try:
            with self._lock:
                self._save_timer = None
                # 应用待写入的变更
                if self._pending_changes:
                    logger.debug(
                        f"应用 {len(self._pending_changes)} 个待写入的配置变更"
                    )
                    for key, value in self._pending_changes.items():
                        self._set_config_value(key, value)
                    self._pending_changes.clear()

                # 执行实际保存
                self._save_config_immediate()
                self._last_save_time = time.monotonic()
                logger.debug("延迟配置保存完成")
        except Exception as e:
            logger.error(f"延迟保存配置失败: {e}", exc_info=True)

    def _set_config_value(self, key: str, value: Any):
        """内部方法：设置配置值（不触发保存，自动创建中间路径）"""
        keys = key.split(".")
        config = self._config

        # 导航到目标位置
        for k in keys[:-1]:
            if k not in config:
                config[k] = {}
            config = config[k]

        # 设置值
        config[keys[-1]] = value

    def _save_config(self):
        """触发延迟保存（通过 _schedule_save 调度）"""
        self._schedule_save()

    def _save_config_immediate(self):
        """原子写入配置文件（tempfile + os.replace），防止崩溃导致文件截断/损坏"""
        try:
            self.config_file.parent.mkdir(parents=True, exist_ok=True)

            if self._is_toml_file() and self._original_content:
                content = self._save_toml_with_comments(self._config)
            elif self._is_toml_file():
                content = tomlkit.dumps(tomlkit.item(self._config))
            else:
                content = json.dumps(self._config, indent=2, ensure_ascii=False)

            # 保留原文件权限（mkstemp 默认 0o600，可能不同于原文件）
            orig_mode = None
            if hasattr(os, "fchmod"):
                try:
                    orig_mode = os.stat(str(self.config_file)).st_mode
                except OSError:
                    pass

            fd, tmp_path = tempfile.mkstemp(
                suffix=".tmp",
                dir=str(self.config_file.parent),
            )
            try:
                with os.fdopen(fd, "w", encoding="utf-8") as f:
                    if orig_mode is not None:
                        os.fchmod(f.fileno(), orig_mode)
                    f.write(content)
                    f.flush()
                    os.fsync(f.fileno())
                os.replace(tmp_path, str(self.config_file))
            except BaseException:
                try:
                    os.unlink(tmp_path)
                except OSError:
                    pass
                raise

            self._original_content = content
            logger.debug(f"配置文件已原子写入: {self.config_file}")

            self._validate_saved_config()
            # 更新 mtime 缓存，避免文件监听器将本次写入误判为外部变更
            self._update_file_mtime()

        except Exception as e:
            logger.error(f"保存配置文件失败: {e}", exc_info=True)
            raise

    def _validate_saved_config(self):
        """验证保存的配置文件格式和结构是否正确"""
        try:
            with open(self.config_file, encoding="utf-8") as f:
                content = f.read()

            parsed_config = self._parse_config_content(content)

            # 额外验证：检查是否存在重复的数组元素（格式损坏的标志）
            self._validate_config_structure(parsed_config, content)

            logger.debug("配置文件验证通过")
        except Exception as e:
            logger.error(f"配置文件验证失败: {e}", exc_info=True)
            raise

    def _validate_config_structure(self, parsed_config: dict[str, Any], content: str):
        """验证配置结构完整性（network_security 格式等）

        TOML 解析器会自动拒绝重复键，此处仅对 JSON 降级格式做额外校验。
        """
        if not self._is_toml_file():
            lines_list = content.splitlines()
            array_definitions: dict[str, int] = {}
            for i, line in enumerate(lines_list):
                stripped = line.strip()
                if not stripped or stripped.startswith("#"):
                    continue
                if '"allowed_networks"' in stripped and "[" in stripped:
                    if "allowed_networks" in array_definitions:
                        raise ConfigValidationError(
                            f"配置文件格式损坏：重复的数组定义在第{i + 1}行"
                        )
                    array_definitions["allowed_networks"] = i + 1
                if '"blocked_ips"' in stripped and "[" in stripped:
                    if "blocked_ips" in array_definitions:
                        raise ConfigValidationError(
                            f"配置文件格式损坏：重复的数组定义在第{i + 1}行"
                        )
                    array_definitions["blocked_ips"] = i + 1

        # 验证network_security配置（如果存在）应该格式正确
        if "network_security" in parsed_config:
            ns_config = parsed_config["network_security"]
            if not isinstance(ns_config, dict):
                raise ConfigValidationError("network_security 配置段必须是 object")
            if "allowed_networks" in ns_config:
                allowed_networks = ns_config["allowed_networks"]
                if not isinstance(allowed_networks, list):
                    raise ConfigValidationError(
                        "network_security.allowed_networks 应该是数组类型"
                    )

                # 检查数组元素是否有效
                for network in allowed_networks:
                    if not isinstance(network, str):
                        raise ConfigValidationError(
                            f"network_security.allowed_networks 包含无效元素: {network}"
                        )

        logger.debug("配置文件结构验证通过")

    def get(self, key: str, default: Any = None) -> Any:
        """获取配置值（支持点号分隔的嵌套键如 'notification.sound_volume'，线程安全）"""
        # 说明：为避免多锁交错导致的竞态/死锁，这里统一使用 _lock 保护共享状态
        with self._lock:
            self._last_access_time = time.monotonic()
            keys = key.split(".")
            value = self._config
            try:
                for k in keys:
                    value = value[k]
                return value
            except (KeyError, TypeError):
                return default

    def set(self, key: str, value: Any, save: bool = True) -> None:
        """设置配置值（支持嵌套键，自动创建中间路径，值变化检测，可选延迟保存）"""
        # network_security 特殊处理：必须走专用更新/落盘路径，避免写入内存但无法持久化
        if key == "network_security":
            if not isinstance(value, dict):
                raise ConfigValidationError("network_security 必须是 object（dict）")
            self.set_network_security_config(cast(dict[str, Any], value), save=save)
            return
        if key.startswith("network_security."):
            field = key[len("network_security.") :]
            if not field or "." in field:
                raise ConfigValidationError(
                    "仅支持设置一级字段：network_security.<field>"
                )
            self.update_network_security_config({field: value}, save=save)
            return

        changed = False
        with self._lock:
            self._last_access_time = time.monotonic()

            # 性能优化：检查当前值是否与新值相同
            current_value = self.get(key)
            if current_value == value:
                logger.debug(
                    f"配置值未变化，跳过更新: {key} = {_sanitize_config_value_for_log(key, value)}"
                )
                return

            # 性能优化：使用缓冲机制
            if save:
                # 将变更添加到待写入队列
                self._pending_changes[key] = value
                # 立即更新内存中的配置
                self._set_config_value(key, value)
                # 调度延迟保存
                self._save_config()
            else:
                # 直接更新内存中的配置，不保存到文件
                self._set_config_value(key, value)
                # 清除 pending 中的同 key 旧值，防止 _delayed_save 回写覆盖
                self._pending_changes.pop(key, None)

            # 【缓存优化】失效相关 section 缓存，避免 get_section() 返回旧值
            section = key.split(".")[0] if key else ""
            if section == "network_security":
                # network_security 有独立缓存层，直接清空所有缓存更稳妥
                self.invalidate_all_caches()
            elif section:
                self.invalidate_section_cache(section)
            else:
                self.invalidate_all_caches()

            changed = True
            logger.debug(
                f"配置已更新: {key} = {_sanitize_config_value_for_log(key, value)}"
            )

        # 【热更新】配置在内存中更新后，触发回调通知其他模块（在锁外执行，避免死锁）
        if changed:
            try:
                self._trigger_config_change_callbacks()
            except Exception as e:
                logger.debug(f"触发配置变更回调失败（忽略）: {e}")

    def update(self, updates: dict[str, Any], save: bool = True) -> None:
        """批量更新配置（仅处理变化项，合并为一次延迟保存，原子操作）"""
        # network_security 特殊处理：先剥离并走专用更新/落盘路径，避免进入 _config/_pending_changes
        network_security_updates: dict[str, Any] = {}
        non_ns_updates: dict[str, Any] = {}
        for k, v in (updates or {}).items():
            if k == "network_security" and isinstance(v, dict):
                # 视为整段覆盖（仍会被验证与归一化）
                network_security_updates.update(cast(dict[str, Any], v))
            elif isinstance(k, str) and k.startswith("network_security."):
                field = k[len("network_security.") :]
                if field and "." not in field:
                    network_security_updates[field] = v
                else:
                    raise ConfigValidationError(
                        "仅支持更新一级字段：network_security.<field>"
                    )
            else:
                non_ns_updates[k] = v

        if network_security_updates:
            self.update_network_security_config(network_security_updates, save=save)
            # 若仅更新 network_security，则无需走通用 update 流程
            if not non_ns_updates:
                return

        changed_sections: set[str] = set()
        changed = False
        with self._lock:
            self._last_access_time = time.monotonic()

            # 性能优化：过滤出真正有变化的配置项
            actual_changes = {}
            for key, value in non_ns_updates.items():
                current_value = self.get(key)
                if current_value != value:
                    actual_changes[key] = value

            if not actual_changes:
                logger.debug("批量更新中没有配置变化，跳过保存")
                return

            # 性能优化：使用批量缓冲机制
            if save:
                # 将所有变更添加到待写入队列
                self._pending_changes.update(actual_changes)
                # 立即更新内存中的配置
                for key, value in actual_changes.items():
                    self._set_config_value(key, value)
                    logger.debug(
                        f"配置已更新: {key} = {_sanitize_config_value_for_log(key, value)}"
                    )
                # 调度延迟保存（只调度一次）
                self._save_config()
            else:
                # 直接更新内存中的配置，不保存到文件
                for key, value in actual_changes.items():
                    self._set_config_value(key, value)
                    # 清除 pending 中的同 key 旧值，防止 _delayed_save 回写覆盖
                    self._pending_changes.pop(key, None)
                    logger.debug(
                        f"配置已更新: {key} = {_sanitize_config_value_for_log(key, value)}"
                    )

            # 【缓存优化】失效涉及到的 section 缓存，避免 get_section() 返回旧值
            for changed_key in actual_changes:
                section = changed_key.split(".")[0] if changed_key else ""
                if section:
                    changed_sections.add(section)

            if "network_security" in changed_sections or not changed_sections:
                self.invalidate_all_caches()
            else:
                for section in changed_sections:
                    self.invalidate_section_cache(section)

            changed = True
            logger.debug(f"批量更新完成，共更新 {len(actual_changes)} 个配置项")

        # 【热更新】配置在内存中更新后，触发回调通知其他模块（在锁外执行，避免死锁）
        if changed:
            try:
                self._trigger_config_change_callbacks()
            except Exception as e:
                logger.debug(f"触发配置变更回调失败（忽略）: {e}")

    def force_save(self) -> None:
        """强制立即保存配置文件（取消延迟保存，应用所有待保存变更）"""
        with self._lock:
            # 取消延迟保存定时器
            if self._save_timer is not None:
                self._save_timer.cancel()
                self._save_timer = None

            # 应用所有待写入的变更
            if self._pending_changes:
                logger.debug(
                    f"强制保存：应用 {len(self._pending_changes)} 个待写入的配置变更"
                )
                for key, value in self._pending_changes.items():
                    self._set_config_value(key, value)
                self._pending_changes.clear()

            # 立即保存
            self._save_config_immediate()
            self._last_save_time = time.monotonic()
            logger.debug("强制配置保存完成")

    def get_section(self, section: str, use_cache: bool = True) -> dict[str, Any]:
        """获取配置段的深拷贝（带 Pydantic 校验、缓存优化，network_security 特殊处理）"""
        import copy

        with self._lock:
            current_time = time.monotonic()

            if section == "network_security":
                return copy.deepcopy(self.get_network_security_config())

            if use_cache and section in self._section_cache:
                cache_time = self._section_cache_time.get(section, 0)
                if current_time - cache_time < self._section_cache_ttl:
                    self._cache_stats["hits"] += 1
                    logger.debug(f"缓存命中: section={section}")
                    return copy.deepcopy(self._section_cache[section])

            self._cache_stats["misses"] += 1
            raw = self.get(section, {})
            result = self._validate_section(section, raw)

            self._section_cache[section] = result
            self._section_cache_time[section] = current_time

            return copy.deepcopy(result)

    @staticmethod
    def _validate_section(section: str, raw: Any) -> dict[str, Any]:
        """通过 Pydantic 模型校验配置段（类型强转 + 钳位），失败时降级返回原始数据"""
        if not isinstance(raw, dict):
            raw = {}
        model_cls = SECTION_MODELS.get(section)
        if model_cls is None:
            return dict(raw)
        try:
            return model_cls.model_validate(raw).model_dump()
        except Exception as e:
            logger.warning(f"配置段 '{section}' Pydantic 校验失败，使用原始值: {e}")
            return dict(raw)

    def update_section(
        self, section: str, updates: dict[str, Any], save: bool = True
    ) -> None:
        """更新配置段（检测变化，触发回调，可选延迟保存）"""
        if section == "network_security":
            if not isinstance(updates, dict):
                raise ConfigValidationError("network_security 更新必须是 dict")
            self.update_network_security_config(updates, save=save)
            return

        changed = False
        with self._lock:
            current_section = self.get_section(section)

            # 检查是否有任何值真的发生了变化
            has_changes = False
            for key, new_value in updates.items():
                current_value = current_section.get(key)
                if current_value != new_value:
                    has_changes = True
                    full_key = f"{section}.{key}"
                    logger.debug(
                        f"配置项 '{full_key}' 发生变化: "
                        f"{_sanitize_config_value_for_log(full_key, current_value)} -> "
                        f"{_sanitize_config_value_for_log(full_key, new_value)}"
                    )

            if not has_changes:
                logger.debug(f"配置段 '{section}' 未发生变化，跳过保存")
                return

            # 应用更新
            current_section.update(updates)

            # 直接更新配置并保存，避免重复的值比较
            keys = section.split(".")
            config = self._config
            for k in keys[:-1]:
                if k not in config:
                    config[k] = {}
                config = config[k]
            config[keys[-1]] = current_section

            if save:
                self._save_config()

            # 【缓存优化】失效该 section 的缓存
            self.invalidate_section_cache(section)

            changed = True
            logger.debug(f"配置段已更新: {section}")

        # 【热更新】配置段更新后触发回调（在锁外执行，避免死锁）
        if changed:
            try:
                self._trigger_config_change_callbacks()
            except Exception as e:
                logger.debug(f"触发配置变更回调失败（忽略）: {e}")

    def reload(self) -> None:
        """从磁盘重新加载配置文件（覆盖内存配置，失效缓存）"""
        logger.info("重新加载配置文件")
        self._load_config()
        # 【缓存优化】重新加载后失效所有缓存
        self.invalidate_all_caches()

    # ========================================================================
    # 缓存管理方法
    # ========================================================================

    def invalidate_section_cache(self, section: str) -> None:
        """失效指定配置段的缓存"""
        with self._lock:
            if section in self._section_cache:
                del self._section_cache[section]
                self._section_cache_time.pop(section, None)
                self._cache_stats["invalidations"] += 1
                logger.debug(f"已失效 section 缓存: {section}")

    def invalidate_all_caches(self) -> None:
        """清空所有配置缓存"""
        with self._lock:
            # 清空 section 缓存
            invalidated_count = len(self._section_cache)
            self._section_cache.clear()
            self._section_cache_time.clear()

            # 清空 network_security 缓存
            self._network_security_cache = None
            self._network_security_cache_time = 0

            self._cache_stats["invalidations"] += invalidated_count + 1
            logger.debug(f"已失效所有缓存 (共 {invalidated_count + 1} 个)")

    def get_cache_stats(self) -> dict[str, Any]:
        """获取缓存统计（命中/未命中/失效次数、命中率等）"""
        with self._lock:
            total = self._cache_stats["hits"] + self._cache_stats["misses"]
            hit_rate = self._cache_stats["hits"] / total if total > 0 else 0.0

            return {
                **self._cache_stats,
                "hit_rate": round(hit_rate, 4),
                "section_cache_size": len(self._section_cache),
                "network_security_cached": self._network_security_cache is not None,
            }

    def reset_cache_stats(self) -> None:
        """重置缓存统计信息"""
        with self._lock:
            self._cache_stats = {
                "hits": 0,
                "misses": 0,
                "invalidations": 0,
            }
            logger.debug("已重置缓存统计")

    def set_cache_ttl(
        self,
        section_ttl: float | None = None,
        network_security_ttl: float | None = None,
    ) -> None:
        """设置缓存有效期（TTL）"""
        with self._lock:
            if section_ttl is not None:
                self._section_cache_ttl = max(0.1, section_ttl)  # 最小 0.1 秒
                logger.debug(f"section 缓存 TTL 已设置为: {self._section_cache_ttl}s")

            if network_security_ttl is not None:
                self._network_security_cache_ttl = max(
                    1.0, network_security_ttl
                )  # 最小 1 秒
                logger.debug(
                    f"network_security 缓存 TTL 已设置为: {self._network_security_cache_ttl}s"
                )

    def get_all(self) -> dict[str, Any]:
        """获取所有配置的深拷贝（不含 network_security），防止外部修改内部状态"""
        import copy

        with self._lock:
            data = copy.deepcopy(self._config)
            return self._exclude_network_security(data)

    @staticmethod
    def _coerce_bool(value: Any, default: bool = True) -> bool:
        """将常见输入转换为 bool（用于配置兼容性）"""
        try:
            if isinstance(value, bool):
                return value
            if value is None:
                return default
            if isinstance(value, (int, float)):
                return bool(value)
            if isinstance(value, str):
                s = value.strip().lower()
                if s in ("true", "1", "yes", "y", "on"):
                    return True
                if s in ("false", "0", "no", "n", "off"):
                    return False
                return default
            return bool(value)
        except Exception:
            return default

    # network_security 方法通过 NetworkSecurityMixin 提供（config_modules/network_security.py）

    # ========================================================================
    # 类型安全的配置获取方法
    # ========================================================================

    def get_typed(
        self,
        key: str,
        default: Any,
        value_type: type,
        min_val: Any | None = None,
        max_val: Any | None = None,
    ) -> Any:
        """获取配置值，带类型转换和边界验证"""
        from config_utils import clamp_value

        raw_value = self.get(key, default)

        try:
            # 布尔类型特殊处理
            if value_type is bool:
                if isinstance(raw_value, bool):
                    return raw_value
                if isinstance(raw_value, str):
                    return raw_value.lower() in ("true", "1", "yes", "on")
                return bool(raw_value)

            # 其他类型转换
            converted = value_type(raw_value)

            # 边界验证（仅对数值类型）
            if value_type in (int, float) and (
                min_val is not None or max_val is not None
            ):
                if min_val is not None and max_val is not None:
                    return clamp_value(converted, min_val, max_val, key)
                elif min_val is not None:
                    return max(converted, min_val)
                elif max_val is not None:
                    return min(converted, max_val)

            return converted

        except (ValueError, TypeError) as e:
            logger.warning(f"配置 '{key}' 类型转换失败: {e}，使用默认值 {default}")
            return default

    def get_int(
        self,
        key: str,
        default: int = 0,
        min_val: int | None = None,
        max_val: int | None = None,
    ) -> int:
        """获取整数配置值"""
        return cast(int, self.get_typed(key, default, int, min_val, max_val))

    def get_float(
        self,
        key: str,
        default: float = 0.0,
        min_val: float | None = None,
        max_val: float | None = None,
    ) -> float:
        """获取浮点数配置值"""
        return cast(float, self.get_typed(key, default, float, min_val, max_val))

    def get_bool(self, key: str, default: bool = False) -> bool:
        """获取布尔配置值"""
        return cast(bool, self.get_typed(key, default, bool))

    def get_str(
        self,
        key: str,
        default: str = "",
        max_length: int | None = None,
    ) -> str:
        """获取字符串配置值（可选截断）"""
        from config_utils import truncate_string

        value = cast(str, self.get_typed(key, default, str))
        if max_length is not None:
            return truncate_string(value, max_length, key, default=default)
        return value

    # 文件监听方法通过 FileWatcherMixin 提供（config_modules/file_watcher.py）
    # 配置导出/导入方法通过 IOOperationsMixin 提供（config_modules/io_operations.py）


# 全局配置管理器实例
config_manager = ConfigManager()

# 【资源生命周期】进程退出时尽量清理后台资源（文件监听/Timer）
# - 避免测试环境出现“退出卡住/资源未释放”类问题
# - shutdown() 本身幂等，重复调用安全
import atexit  # noqa: E402


def _shutdown_global_config_manager():
    try:
        config_manager.shutdown()
    except Exception:
        # 退出阶段不再抛异常
        pass


atexit.register(_shutdown_global_config_manager)


def get_config() -> ConfigManager:
    """获取全局配置管理器实例（自动启动文件监听）"""
    # 【配置热更新】默认启用文件监听（2 秒轮询，按你的选择 A + C）
    # 目的：外部编辑 config.toml 后无需重启即可生效
    try:
        if not config_manager.is_file_watcher_running:
            config_manager.start_file_watcher(interval=2.0)
    except Exception:
        # 配置系统属于基础设施：监听启动失败不应影响主流程
        pass

    return config_manager



"""安全策略 Mixin — 从 WebFeedbackUI 提取。

提供 IP 访问控制、CSP 安全头注入、网络安全配置加载等方法，
由 WebFeedbackUI 通过 MRO 继承。
"""

from __future__ import annotations

import secrets
from ipaddress import AddressValueError, ip_address, ip_network
from typing import TYPE_CHECKING, Any

from flask import Response, abort, g, request
from flask.typing import ResponseReturnValue

from config_manager import get_config
from enhanced_logging import EnhancedLogger
from web_ui_validators import validate_network_security_config

if TYPE_CHECKING:
    from flask import Flask

logger = EnhancedLogger(__name__)


class SecurityMixin:
    """IP 访问控制 + HTTP 安全头 + CSP nonce 管理。"""

    if TYPE_CHECKING:
        app: Flask
        network_security_config: dict[str, Any]
        host: str

    # ------------------------------------------------------------------
    # 安全头 & 访问控制 hook
    # ------------------------------------------------------------------

    def setup_security_headers(self) -> None:
        """注册 before_request / after_request 钩子：IP 访问控制 + 安全头注入。"""

        @self.app.before_request
        def check_ip_and_generate_nonce() -> ResponseReturnValue | None:
            client_ip = self._get_request_client_ip(request.environ)
            if not self._is_ip_allowed(client_ip):
                logger.warning(f"拒绝来自 {client_ip} 的访问请求")
                abort(403)
            g.csp_nonce = secrets.token_urlsafe(16)

        @self.app.after_request
        def add_security_headers(response: Response) -> Response:
            nonce = getattr(g, "csp_nonce", "")
            response.headers["Content-Security-Policy"] = (
                "default-src 'self'; "
                f"script-src 'self' 'nonce-{nonce}'; "
                "style-src 'self' 'unsafe-inline'; "
                "img-src 'self' data: blob:; "
                "font-src 'self' data:; "
                "connect-src 'self'; "
                "worker-src 'self'; "
                "frame-ancestors 'none'; "
                "base-uri 'self'; "
                "object-src 'none'"
            )
            response.headers["X-Frame-Options"] = "DENY"
            response.headers["X-Content-Type-Options"] = "nosniff"
            response.headers["X-XSS-Protection"] = "1; mode=block"
            response.headers["Referrer-Policy"] = "strict-origin-when-cross-origin"
            response.headers["Permissions-Policy"] = (
                "geolocation=(), microphone=(), camera=(), "
                "payment=(), usb=(), magnetometer=(), gyroscope=()"
            )

            path = request.path
            if path.startswith(("/static/js/", "/static/css/")):
                if request.args.get("v"):
                    response.headers["Cache-Control"] = (
                        "public, max-age=31536000, immutable"
                    )
                else:
                    response.headers["Cache-Control"] = "public, max-age=86400"
            elif path.startswith(("/static/lottie/", "/fonts/")):
                response.headers["Cache-Control"] = "public, max-age=2592000, immutable"
            elif path.startswith("/sounds/") or (
                path.startswith("/icons/") and not path.endswith(".ico")
            ):
                response.headers["Cache-Control"] = "public, max-age=604800"

            return response

    # ------------------------------------------------------------------
    # CSP nonce
    # ------------------------------------------------------------------

    def _get_csp_nonce(self) -> str:
        """获取当前请求的 CSP nonce；非请求上下文时生成临时随机值。"""
        from flask import has_request_context

        try:
            if has_request_context():
                return getattr(g, "csp_nonce", secrets.token_urlsafe(16))
        except RuntimeError:
            pass
        return secrets.token_urlsafe(16)

    # ------------------------------------------------------------------
    # 网络安全配置
    # ------------------------------------------------------------------

    def _load_network_security_config(self) -> dict:
        """加载并验证 network_security 配置，失败时返回默认值。"""
        try:
            config_mgr = get_config()
            raw_config = config_mgr.get_section("network_security")
            return validate_network_security_config(raw_config)
        except Exception as e:
            logger.warning(f"无法加载网络安全配置，使用默认配置: {e}", exc_info=True)
            return validate_network_security_config({})

    # ------------------------------------------------------------------
    # IP 访问控制
    # ------------------------------------------------------------------

    @staticmethod
    def _normalize_addr(addr_str: str):
        """规范化 IP 地址，将 IPv4-mapped IPv6 转换为纯 IPv4。"""
        addr = ip_address(addr_str)
        if hasattr(addr, "ipv4_mapped") and addr.ipv4_mapped:
            addr = addr.ipv4_mapped
        return addr

    def _is_ip_allowed(self, client_ip: str) -> bool:
        """根据白名单/黑名单验证客户端 IP 是否允许访问。"""
        cfg = (
            self.network_security_config
            if isinstance(self.network_security_config, dict)
            else {}
        )
        if not cfg.get("access_control_enabled", True):
            return True

        try:
            client_addr = self._normalize_addr(client_ip)

            blocked_ips = cfg.get("blocked_ips", [])
            for blocked_entry in blocked_ips:
                try:
                    if "/" in blocked_entry:
                        if client_addr in ip_network(blocked_entry, strict=False):
                            logger.warning(
                                f"IP {client_ip} 在黑名单网段 {blocked_entry} 中，拒绝访问"
                            )
                            return False
                    else:
                        if client_addr == self._normalize_addr(blocked_entry):
                            logger.warning(f"IP {client_ip} 在黑名单中，拒绝访问")
                            return False
                except (AddressValueError, ValueError, TypeError):
                    continue

            allowed_networks = cfg.get("allowed_networks", ["127.0.0.0/8", "::1/128"])
            for network_str in allowed_networks:
                try:
                    if "/" in network_str:
                        if client_addr in ip_network(network_str, strict=False):
                            return True
                    else:
                        if client_addr == self._normalize_addr(network_str):
                            return True
                except (AddressValueError, ValueError, TypeError) as e:
                    logger.warning(f"无效的网络配置 {network_str}: {e}")
                    continue

            logger.warning(f"IP {client_ip} 不在允许的网络范围内，拒绝访问")
            return False

        except AddressValueError as e:
            logger.warning(f"无效的IP地址 {client_ip}: {e}")
            return False

    # ------------------------------------------------------------------
    # X-Forwarded-For / 客户端 IP
    # ------------------------------------------------------------------

    @staticmethod
    def _parse_forwarded_for(forwarded_for: str) -> str:
        """从 X-Forwarded-For 中提取首个客户端 IP。"""
        if not forwarded_for:
            return ""
        return forwarded_for.split(",")[0].strip()

    @classmethod
    def _should_trust_forwarded_for(cls, remote_addr: str) -> bool:
        """仅信任来自本机反向代理的 X-Forwarded-For。"""
        if not remote_addr:
            return False
        try:
            return cls._normalize_addr(remote_addr).is_loopback
        except (AddressValueError, ValueError):
            return False

    def _get_request_client_ip(self, environ: dict[str, Any]) -> str:
        """获取用于访问控制的客户端 IP。"""
        remote_addr = str(environ.get("REMOTE_ADDR", "")).strip()
        forwarded_for = str(environ.get("HTTP_X_FORWARDED_FOR", "")).strip()

        if self._should_trust_forwarded_for(remote_addr):
            forwarded_ip = self._parse_forwarded_for(forwarded_for)
            if forwarded_ip:
                return forwarded_ip

        return remote_addr



#!/usr/bin/env python3
"""通知领域模型：枚举与事件结构（避免 manager/provider 循环依赖）。"""

import time
from enum import Enum
from typing import Any

from pydantic import BaseModel, ConfigDict, Field, field_validator


class NotificationType(Enum):
    """通知类型枚举：WEB(浏览器)、SOUND(声音)、BARK(iOS推送)、SYSTEM(系统)"""

    WEB = "web"
    SOUND = "sound"
    BARK = "bark"
    SYSTEM = "system"


class NotificationTrigger(Enum):
    """通知触发时机：立即/延迟/重复/反馈收到/错误"""

    IMMEDIATE = "immediate"
    DELAYED = "delayed"
    REPEAT = "repeat"
    FEEDBACK_RECEIVED = "feedback_received"
    ERROR = "error"


class NotificationPriority(Enum):
    """通知优先级：用于路由/降级/节流（阶段 A 先完成数据结构与可观测性）。"""

    LOW = "low"
    NORMAL = "normal"
    HIGH = "high"
    URGENT = "urgent"


class NotificationEvent(BaseModel):
    """通知事件 - 封装一次通知的标题/消息/类型/触发时机/重试信息。"""

    model_config = ConfigDict(validate_assignment=True)

    id: str
    title: str
    message: str
    trigger: NotificationTrigger
    types: list[NotificationType] = Field(default_factory=list)
    metadata: dict[str, Any] = Field(default_factory=dict)
    timestamp: float = Field(default_factory=time.time)

    @field_validator("metadata", mode="before")
    @classmethod
    def coerce_none_metadata(cls, v: Any) -> dict[str, Any]:
        if v is None:
            return {}
        return v

    retry_count: int = 0
    max_retries: int = 3
    priority: NotificationPriority = NotificationPriority.NORMAL
    source: str | None = None
    dedupe_key: str | None = None



"""通知路由 Mixin — Bark 测试、新任务通知、通知配置读写、反馈提示语。"""

from __future__ import annotations

import time
from typing import TYPE_CHECKING, Any

from flask import jsonify, request
from flask.typing import ResponseReturnValue

from config_manager import get_config
from config_utils import clamp_value
from enhanced_logging import EnhancedLogger
from i18n import msg

try:
    from notification_manager import (
        NotificationEvent,
        NotificationTrigger,
        NotificationType,
        notification_manager,
    )
    from notification_providers import BarkNotificationProvider

    NOTIFICATION_AVAILABLE = True
except ImportError:
    NOTIFICATION_AVAILABLE = False

if TYPE_CHECKING:
    from flask import Flask

logger = EnhancedLogger(__name__)


class NotificationRoutesMixin:
    """提供 5 个通知相关 API 路由，由 WebFeedbackUI 通过 MRO 继承。"""

    if TYPE_CHECKING:
        app: Flask
        limiter: Any

    def _setup_notification_routes(self) -> None:
        @self.app.route("/api/test-bark", methods=["POST"])
        def test_bark_notification() -> ResponseReturnValue:
            """测试 Bark 推送通知
            ---
            tags:
              - Notification
            consumes:
              - application/json
            parameters:
              - in: body
                name: body
                required: true
                schema:
                  type: object
                  required:
                    - bark_device_key
                  properties:
                    bark_url:
                      type: string
                      default: "https://api.day.app/push"
                    bark_device_key:
                      type: string
                      description: Bark 设备密钥
                    bark_icon:
                      type: string
                    bark_action:
                      type: string
                      default: "none"
            responses:
              200:
                description: 测试通知发送成功
                schema:
                  type: object
                  properties:
                    status:
                      type: string
                      example: success
                    message:
                      type: string
              400:
                description: 设备密钥为空
              500:
                description: 发送失败或通知系统不可用
            """
            try:
                data = request.json or {}
                bark_url = data.get("bark_url", "https://api.day.app/push")
                bark_device_key = data.get("bark_device_key", "")
                bark_icon = data.get("bark_icon", "")
                bark_action = data.get("bark_action", "none")

                if not bark_device_key:
                    return jsonify(
                        {"status": "error", "message": msg("notify.deviceKeyEmpty")}
                    ), 400

                try:
                    if not NOTIFICATION_AVAILABLE:
                        raise ImportError(msg("notify.systemUnavailable"))

                    class TempConfig:
                        def __init__(self) -> None:
                            self.bark_enabled = True
                            self.bark_url = bark_url
                            self.bark_device_key = bark_device_key
                            self.bark_icon = bark_icon
                            self.bark_action = bark_action

                    temp_config = TempConfig()
                    bark_provider = BarkNotificationProvider(temp_config)

                    test_event = NotificationEvent(
                        id=f"test_bark_{int(time.time())}",
                        title="AI Intervention Agent 测试",
                        message="这是一个 Bark 通知测试，如果收到此消息，说明配置正确。",
                        trigger=NotificationTrigger.IMMEDIATE,
                        types=[NotificationType.BARK],
                        metadata={"test": True},
                    )

                    success = bark_provider.send(test_event)

                    if success:
                        return jsonify(
                            {
                                "status": "success",
                                "message": msg("notify.testSuccess"),
                            }
                        )
                    else:
                        bark_error = None
                        try:
                            if isinstance(test_event.metadata, dict):
                                bark_error = test_event.metadata.get("bark_error")
                        except Exception:
                            bark_error = None

                        if isinstance(bark_error, dict) and bark_error.get("detail"):
                            detail = str(bark_error.get("detail"))[:300]
                            status_code = bark_error.get("status_code")
                            status_hint = (
                                f"(HTTP {status_code}) " if status_code else ""
                            )
                            return jsonify(
                                {
                                    "status": "error",
                                    "message": msg(
                                        "notify.sendFailedDetail",
                                        detail=f"{status_hint}{detail}",
                                    ),
                                }
                            ), 500
                        return jsonify(
                            {
                                "status": "error",
                                "message": msg("notify.sendFailedConfig"),
                            }
                        ), 500

                except ImportError as e:
                    logger.error(f"导入通知系统失败: {e}", exc_info=True)
                    return jsonify(
                        {"status": "error", "message": msg("notify.systemUnavailable")}
                    ), 500

            except Exception as e:
                logger.error(f"Bark 测试通知失败: {e}", exc_info=True)
                return jsonify(
                    {"status": "error", "message": msg("notify.testFailed")}
                ), 500

        @self.app.route("/api/notify-new-tasks", methods=["POST"])
        def notify_new_tasks() -> ResponseReturnValue:
            """触发新任务 Bark 推送通知
            ---
            tags:
              - Notification
            consumes:
              - application/json
            parameters:
              - in: body
                name: body
                schema:
                  type: object
                  properties:
                    count:
                      type: integer
                      description: 新任务数量
                    taskIds:
                      type: array
                      items:
                        type: string
                      description: 新任务 ID 列表
            responses:
              200:
                description: 通知已触发或已跳过（优雅降级）
                schema:
                  type: object
                  properties:
                    status:
                      type: string
                      enum: [success, skipped]
                    event_id:
                      type: string
                    message:
                      type: string
              500:
                description: 触发失败
            """
            try:
                data = request.json or {}

                raw_task_ids = data.get("taskIds", data.get("task_ids", []))
                task_ids: list[str] = []
                if isinstance(raw_task_ids, list):
                    for item in raw_task_ids[:50]:
                        if item is None:
                            continue
                        s = str(item)
                        if not s:
                            continue
                        task_ids.append(s[:200])

                raw_count = data.get("count")
                count = 0
                if isinstance(raw_count, (int, float)):
                    try:
                        count = int(raw_count)
                    except Exception:
                        count = 0
                if count <= 0:
                    count = len(task_ids)

                if count <= 0:
                    return jsonify(
                        {"status": "skipped", "message": msg("notify.countZero")}
                    )

                if not NOTIFICATION_AVAILABLE:
                    return jsonify(
                        {"status": "skipped", "message": msg("notify.systemDegraded")}
                    )

                try:
                    notification_manager.refresh_config_from_file()
                except Exception:
                    pass

                cfg = getattr(notification_manager, "config", None)
                if not cfg or not getattr(cfg, "enabled", True):
                    return jsonify(
                        {"status": "skipped", "message": msg("notify.globalOff")}
                    )

                if not getattr(cfg, "bark_enabled", False):
                    return jsonify(
                        {"status": "skipped", "message": msg("notify.barkDisabled")}
                    )

                if not getattr(cfg, "bark_device_key", ""):
                    return jsonify(
                        {"status": "skipped", "message": msg("notify.barkKeyEmpty")}
                    )

                title = "AI Intervention Agent"
                message = (
                    f"新任务已添加: {task_ids[0]}"
                    if count == 1 and task_ids
                    else f"收到 {count} 个新任务"
                )

                event_id = notification_manager.send_notification(
                    title=title,
                    message=message,
                    trigger=NotificationTrigger.IMMEDIATE,
                    types=[NotificationType.BARK],
                    metadata={
                        "source": "web_ui",
                        "event": "new_tasks",
                        "count": count,
                        "task_ids": task_ids,
                    },
                    priority="normal",
                )

                if not event_id:
                    return jsonify(
                        {"status": "skipped", "message": msg("notify.notTriggered")}
                    )

                return jsonify({"status": "success", "event_id": event_id})
            except Exception as e:
                logger.error(f"触发新任务通知失败: {e}", exc_info=True)
                return jsonify(
                    {"status": "error", "message": msg("notify.triggerFailed")}
                ), 500

        @self.app.route("/api/update-notification-config", methods=["POST"])
        def update_notification_config() -> ResponseReturnValue:
            """更新通知配置
            ---
            tags:
              - Notification
            consumes:
              - application/json
            parameters:
              - in: body
                name: body
                schema:
                  type: object
                  properties:
                    enabled:
                      type: boolean
                    webEnabled:
                      type: boolean
                    soundEnabled:
                      type: boolean
                    soundVolume:
                      type: integer
                      minimum: 0
                      maximum: 100
                    barkEnabled:
                      type: boolean
                    barkUrl:
                      type: string
                    barkDeviceKey:
                      type: string
                    barkIcon:
                      type: string
                    barkAction:
                      type: string
                    macosNativeEnabled:
                      type: boolean
            responses:
              200:
                description: 配置已更新
                schema:
                  type: object
                  properties:
                    status:
                      type: string
                      example: success
                    message:
                      type: string
              500:
                description: 更新失败
            """
            try:
                data = request.json or {}
                if not isinstance(data, dict):
                    data = {}

                try:
                    if not NOTIFICATION_AVAILABLE:
                        raise ImportError(msg("notify.systemUnavailable"))

                    config_mgr = get_config()
                    notification_config = dict(config_mgr.get_section("notification"))

                    def normalize_sound_volume(raw_value: Any) -> int:
                        try:
                            return int(
                                clamp_value(float(raw_value), 0, 100, "sound_volume")
                            )
                        except (TypeError, ValueError):
                            return int(notification_config.get("sound_volume", 80))

                    def normalize_web_timeout(raw_value: Any) -> int:
                        try:
                            return int(
                                clamp_value(float(raw_value), 1, 600000, "web_timeout")
                            )
                        except (TypeError, ValueError):
                            return int(notification_config.get("web_timeout", 5000))

                    def normalize_string(raw_value: Any) -> str:
                        return "" if raw_value is None else str(raw_value)

                    def first_present(*keys: str) -> tuple[bool, Any]:
                        for key in keys:
                            if key in data:
                                return True, data.get(key)
                        return False, None

                    field_specs = [
                        (("enabled",), "enabled", "enabled", lambda v: v, lambda v: v),
                        (
                            ("webEnabled", "web_enabled"),
                            "web_enabled",
                            "web_enabled",
                            lambda v: v,
                            lambda v: v,
                        ),
                        (
                            ("webIcon", "web_icon"),
                            "web_icon",
                            "web_icon",
                            normalize_string,
                            normalize_string,
                        ),
                        (
                            ("webTimeout", "web_timeout"),
                            "web_timeout",
                            "web_timeout",
                            normalize_web_timeout,
                            normalize_web_timeout,
                        ),
                        (
                            ("autoRequestPermission", "auto_request_permission"),
                            "web_permission_auto_request",
                            "auto_request_permission",
                            lambda v: v,
                            lambda v: v,
                        ),
                        (
                            ("macosNativeEnabled", "macos_native_enabled"),
                            "macos_native_enabled",
                            "macos_native_enabled",
                            lambda v: v,
                            lambda v: v,
                        ),
                        (
                            ("soundEnabled", "sound_enabled"),
                            "sound_enabled",
                            "sound_enabled",
                            lambda v: v,
                            lambda v: v,
                        ),
                        (
                            ("soundFile", "sound_file"),
                            "sound_file",
                            "sound_file",
                            normalize_string,
                            normalize_string,
                        ),
                        (
                            ("soundMute", "sound_mute"),
                            "sound_mute",
                            "sound_mute",
                            lambda v: v,
                            lambda v: v,
                        ),
                        (
                            ("soundVolume", "sound_volume"),
                            "sound_volume",
                            "sound_volume",
                            lambda v: normalize_sound_volume(v) / 100.0,
                            normalize_sound_volume,
                        ),
                        (
                            ("mobileOptimized", "mobile_optimized"),
                            "mobile_optimized",
                            "mobile_optimized",
                            lambda v: v,
                            lambda v: v,
                        ),
                        (
                            ("mobileVibrate", "mobile_vibrate"),
                            "mobile_vibrate",
                            "mobile_vibrate",
                            lambda v: v,
                            lambda v: v,
                        ),
                        (
                            ("barkEnabled", "bark_enabled"),
                            "bark_enabled",
                            "bark_enabled",
                            lambda v: v,
                            lambda v: v,
                        ),
                        (
                            ("barkUrl", "bark_url"),
                            "bark_url",
                            "bark_url",
                            normalize_string,
                            normalize_string,
                        ),
                        (
                            ("barkDeviceKey", "bark_device_key"),
                            "bark_device_key",
                            "bark_device_key",
                            normalize_string,
                            normalize_string,
                        ),
                        (
                            ("barkIcon", "bark_icon"),
                            "bark_icon",
                            "bark_icon",
                            normalize_string,
                            normalize_string,
                        ),
                        (
                            ("barkAction", "bark_action"),
                            "bark_action",
                            "bark_action",
                            normalize_string,
                            normalize_string,
                        ),
                    ]

                    manager_updates: dict[str, Any] = {}
                    changed_keys: list[str] = []
                    for (
                        request_keys,
                        manager_key,
                        config_key,
                        manager_cast,
                        config_cast,
                    ) in field_specs:
                        found, raw_value = first_present(*request_keys)
                        if not found:
                            continue
                        manager_updates[manager_key] = manager_cast(raw_value)
                        notification_config[config_key] = config_cast(raw_value)
                        changed_keys.append(config_key)

                    if not changed_keys:
                        logger.info("通知配置更新请求未包含可识别字段，已忽略")
                        return jsonify(
                            {
                                "status": "success",
                                "message": msg("notify.noUpdateFields"),
                            }
                        )

                    notification_manager.update_config_without_save(**manager_updates)
                    config_mgr.update_section("notification", notification_config)

                    logger.info("通知配置已更新到配置文件和内存")
                    return jsonify(
                        {"status": "success", "message": msg("notify.configUpdated")}
                    )

                except ImportError as e:
                    logger.error(f"导入配置系统失败: {e}", exc_info=True)
                    return jsonify(
                        {"status": "error", "message": msg("notify.configUnavailable")}
                    ), 500

            except Exception as e:
                logger.error(f"更新通知配置失败: {e}", exc_info=True)
                return jsonify(
                    {"status": "error", "message": msg("notify.updateFailed")}
                ), 500

        @self.app.route("/api/get-notification-config", methods=["GET"])
        def get_notification_config() -> ResponseReturnValue:
            """获取当前通知配置
            ---
            tags:
              - Notification
            responses:
              200:
                description: 通知配置
                schema:
                  type: object
                  properties:
                    status:
                      type: string
                      example: success
                    config:
                      type: object
                      description: 完整通知配置项
              500:
                description: 读取配置失败
            """
            try:
                config_mgr = get_config()
                notification_config = config_mgr.get_section("notification")

                return jsonify({"status": "success", "config": notification_config})

            except Exception as e:
                logger.error(f"获取通知配置失败: {e}", exc_info=True)
                return jsonify(
                    {"status": "error", "message": msg("notify.getFailed")}
                ), 500

        @self.app.route("/api/get-feedback-prompts", methods=["GET"])
        def get_feedback_prompts_api() -> ResponseReturnValue:
            """获取反馈提示语配置
            ---
            tags:
              - Feedback
            responses:
              200:
                description: 反馈提示语配置
                schema:
                  type: object
                  properties:
                    status:
                      type: string
                      example: success
                    config:
                      type: object
                      properties:
                        frontend_countdown:
                          type: integer
                          description: 前端倒计时时间（秒）
                        resubmit_prompt:
                          type: string
                          description: 错误/超时时的提示语
                        prompt_suffix:
                          type: string
                          description: 追加到反馈末尾的提示语
                    meta:
                      type: object
                      properties:
                        config_file:
                          type: string
                        override_env:
                          type: string
              500:
                description: 读取配置失败
            """
            try:
                config_mgr = get_config()
                feedback_config = config_mgr.get_section("feedback")

                from config_utils import truncate_string
                from server_config import AUTO_RESUBMIT_TIMEOUT_DEFAULT

                raw_countdown = feedback_config.get(
                    "frontend_countdown",
                    feedback_config.get(
                        "auto_resubmit_timeout",
                        AUTO_RESUBMIT_TIMEOUT_DEFAULT,
                    ),
                )
                try:
                    frontend_countdown = int(raw_countdown)
                except (TypeError, ValueError):
                    frontend_countdown = AUTO_RESUBMIT_TIMEOUT_DEFAULT

                return jsonify(
                    {
                        "status": "success",
                        "config": {
                            "frontend_countdown": frontend_countdown,
                            "resubmit_prompt": truncate_string(
                                feedback_config.get("resubmit_prompt"),
                                500,
                                "feedback.resubmit_prompt",
                                default="请立即调用 interactive_feedback 工具",
                            ),
                            "prompt_suffix": truncate_string(
                                feedback_config.get("prompt_suffix"),
                                500,
                                "feedback.prompt_suffix",
                                default="\n请积极调用 interactive_feedback 工具",
                            ),
                        },
                        "meta": {
                            "config_file": str(config_mgr.config_file.absolute()),
                            "override_env": "AI_INTERVENTION_AGENT_CONFIG_FILE",
                        },
                    }
                )

            except Exception as e:
                logger.error(f"获取反馈提示语配置失败: {e}", exc_info=True)
                return jsonify(
                    {"status": "error", "message": msg("notify.getFailed")}
                ), 500

        @self.app.route("/api/update-feedback-config", methods=["POST"])
        @self.limiter.limit("30 per minute")
        def update_feedback_config() -> ResponseReturnValue:
            """更新反馈配置（倒计时、提示语）
            ---
            tags:
              - Feedback
            consumes:
              - application/json
            parameters:
              - in: body
                name: body
                schema:
                  type: object
                  properties:
                    frontend_countdown:
                      type: integer
                      minimum: 0
                      maximum: 250
                    resubmit_prompt:
                      type: string
                      maxLength: 500
                    prompt_suffix:
                      type: string
                      maxLength: 500
            responses:
              200:
                description: 配置已更新
              400:
                description: 参数无效
              500:
                description: 更新失败
            """
            try:
                data = request.json or {}
                if not isinstance(data, dict):
                    data = {}

                config_mgr = get_config()
                feedback_config = dict(config_mgr.get_section("feedback"))

                from config_utils import truncate_string
                from server_config import (
                    AUTO_RESUBMIT_TIMEOUT_MAX,
                    AUTO_RESUBMIT_TIMEOUT_MIN,
                    PROMPT_SUFFIX_DEFAULT,
                    RESUBMIT_PROMPT_DEFAULT,
                )

                changed_keys: list[str] = []

                if "frontend_countdown" in data:
                    raw = data["frontend_countdown"]
                    try:
                        val = int(raw)
                    except (TypeError, ValueError):
                        return jsonify(
                            {
                                "status": "error",
                                "message": "frontend_countdown 必须为整数",
                            }
                        ), 400
                    if val == 0:
                        feedback_config["frontend_countdown"] = 0
                    else:
                        feedback_config["frontend_countdown"] = int(
                            clamp_value(
                                val,
                                AUTO_RESUBMIT_TIMEOUT_MIN,
                                AUTO_RESUBMIT_TIMEOUT_MAX,
                                "frontend_countdown",
                            )
                        )
                    changed_keys.append("frontend_countdown")

                if "resubmit_prompt" in data:
                    feedback_config["resubmit_prompt"] = truncate_string(
                        data["resubmit_prompt"],
                        500,
                        "feedback.resubmit_prompt",
                        default=RESUBMIT_PROMPT_DEFAULT,
                    )
                    changed_keys.append("resubmit_prompt")

                if "prompt_suffix" in data:
                    feedback_config["prompt_suffix"] = truncate_string(
                        data["prompt_suffix"],
                        500,
                        "feedback.prompt_suffix",
                        default=PROMPT_SUFFIX_DEFAULT,
                    )
                    changed_keys.append("prompt_suffix")

                if not changed_keys:
                    return jsonify(
                        {"status": "success", "message": "无可识别的更新字段"}
                    )

                config_mgr.update_section("feedback", feedback_config)
                logger.info(f"反馈配置已更新: {changed_keys}")

                return jsonify({"status": "success", "message": "反馈配置已更新"})

            except Exception as e:
                logger.error(f"更新反馈配置失败: {e}", exc_info=True)
                return jsonify(
                    {"status": "error", "message": msg("notify.updateFailed")}
                ), 500
